//
//	ReaderMainToolbar.m
//	Reader v2.6.2
//
//	Created by Julius Oklamcak on 2011-07-01.
//	Copyright © 2011-2013 Julius Oklamcak. All rights reserved.
//
//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights to
//	use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
//	of the Software, and to permit persons to whom the Software is furnished to
//	do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in all
//	copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
//	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
//	CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

#import "ReaderConstants.h"
#import "ReaderMainToolbar.h"
#import "ReaderDocument.h"

#import "PRTAnimatingButton.h"

#import <MessageUI/MessageUI.h>

@implementation ReaderMainToolbar { }

#pragma mark Constants

#define BUTTON_X 8.0f
#define BUTTON_Y 8.0f
#define BUTTON_SPACE 20.0f
#define BUTTON_HEIGHT 30.0f
#define BUTTON_WIDTH 40.0f
#define TITLE_HEIGHT 28.0f

#pragma mark Properties

@synthesize delegate;

#pragma mark ReaderMainToolbar instance methods

- (id)initWithFrame:(CGRect)frame
{
	return [self initWithFrame:frame document:nil];
}

- (id)initWithFrame:(CGRect)frame document:(ReaderDocument *)object
{
	assert(object != nil); // Must have a valid ReaderDocument

	if ((self = [super initWithFrame:frame]))
	{
		CGFloat viewWidth = self.bounds.size.width;
		CGFloat titleX = BUTTON_X;
        CGFloat titleWidth = (viewWidth - (titleX + titleX));
		CGFloat leftButtonX = BUTTON_X; // Left button start X position
        
        // Close button
        {
            UIButton *closeButton = [UIButton buttonWithType:UIButtonTypeCustom];
            closeButton.backgroundColor = [UIColor blueColor];
            closeButton.frame = CGRectMake(leftButtonX, BUTTON_Y, BUTTON_WIDTH, BUTTON_HEIGHT);
            [closeButton addTarget:self action:@selector(closeButtonTapped:) forControlEvents:UIControlEventTouchUpInside];
            closeButton.titleLabel.font = [UIFont systemFontOfSize:14.0f];
            closeButton.autoresizingMask = UIViewAutoresizingNone;
            closeButton.exclusiveTouch = YES;
            [self addSubview:closeButton];
            
            leftButtonX += (BUTTON_WIDTH + BUTTON_SPACE);
            titleX += (BUTTON_WIDTH + BUTTON_SPACE);
            titleWidth -= (BUTTON_WIDTH + BUTTON_SPACE);
        }
        
        CGFloat rightButtonX = viewWidth; // Right button start X position
        
        // Publish button
        {
            rightButtonX -= (BUTTON_WIDTH + BUTTON_SPACE);
            
            PRTAnimatingButton *publishButton = [PRTAnimatingButton buttonWithType:UIButtonTypeCustom];
            publishButton.frame = CGRectMake(rightButtonX, BUTTON_Y, BUTTON_WIDTH, BUTTON_HEIGHT);
            publishButton.selectedAnimatingImages = [NSArray arrayWithObjects:
                                                     [UIImage imageNamed:@"wifi_on_0.png"],
                                                     [UIImage imageNamed:@"wifi_on_1.png"],
                                                     [UIImage imageNamed:@"wifi_on_2.png"],
                                                     [UIImage imageNamed:@"wifi_on_3.png"],
                                                     [UIImage imageNamed:@"wifi_on_2.png"],
                                                     [UIImage imageNamed:@"wifi_on_1.png"],
                                                     nil];
            publishButton.deselectedAnimatingImages = [NSArray arrayWithObjects:
                                                       [UIImage imageNamed:@"wifi_off.png"],
                                                       nil];
            [publishButton addTarget:self action:@selector(publishButtonTapped:) forControlEvents:UIControlEventTouchUpInside];
            publishButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin;
            publishButton.exclusiveTouch = YES;
            [self addSubview:publishButton];
            
            titleWidth -= (BUTTON_WIDTH + BUTTON_SPACE);
        }
        
        // Public edit button
        {
            rightButtonX -= (BUTTON_WIDTH + BUTTON_SPACE);
            
            UIButton *editButton = [UIButton buttonWithType:UIButtonTypeCustom];
            editButton.backgroundColor = [UIColor greenColor];
            editButton.frame = CGRectMake(rightButtonX, BUTTON_Y, BUTTON_WIDTH, BUTTON_HEIGHT);
            [editButton addTarget:self action:@selector(publicEditButtonTapped:) forControlEvents:UIControlEventTouchUpInside];
            editButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin;
            editButton.exclusiveTouch = YES;
            [self addSubview:editButton];
            
            titleWidth -= (BUTTON_WIDTH + BUTTON_SPACE);
        }
        
        // Private edit button
        {
            rightButtonX -= (BUTTON_WIDTH + BUTTON_SPACE);
            
            UIButton *editButton = [UIButton buttonWithType:UIButtonTypeCustom];
            editButton.backgroundColor = [UIColor yellowColor];
            editButton.frame = CGRectMake(rightButtonX, BUTTON_Y, BUTTON_WIDTH, BUTTON_HEIGHT);
            [editButton addTarget:self action:@selector(privateEditButtonTapped:) forControlEvents:UIControlEventTouchUpInside];
            editButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin;
            editButton.exclusiveTouch = YES;
            [self addSubview:editButton];
            
            titleWidth -= (BUTTON_WIDTH + BUTTON_SPACE);
        }
        
        // Point button
        {
            rightButtonX -= (BUTTON_WIDTH + BUTTON_SPACE);
            
            UIButton *editButton = [UIButton buttonWithType:UIButtonTypeCustom];
            editButton.backgroundColor = [UIColor purpleColor];
            editButton.frame = CGRectMake(rightButtonX, BUTTON_Y, BUTTON_WIDTH, BUTTON_HEIGHT);
            [editButton addTarget:self action:@selector(pointButtonTapped:) forControlEvents:UIControlEventTouchUpInside];
            editButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin;
            editButton.exclusiveTouch = YES;
            [self addSubview:editButton];
            
            titleWidth -= (BUTTON_WIDTH + BUTTON_SPACE);
        }
        
        // Sync button
        {
            rightButtonX -= (BUTTON_WIDTH + BUTTON_SPACE);
            
            UIButton *editButton = [UIButton buttonWithType:UIButtonTypeCustom];
            editButton.backgroundColor = [UIColor redColor];
            editButton.frame = CGRectMake(rightButtonX, BUTTON_Y, BUTTON_WIDTH, BUTTON_HEIGHT);
            [editButton addTarget:self action:@selector(syncButtonTapped:) forControlEvents:UIControlEventTouchUpInside];
            editButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin;
            editButton.exclusiveTouch = YES;
            [self addSubview:editButton];
            
            titleWidth -= (BUTTON_WIDTH + BUTTON_SPACE);
        }
	}

	return self;
}

- (void)setBookmarkState:(BOOL)state { }

- (void)hideToolbar
{
	if (self.hidden == NO)
	{
		[UIView animateWithDuration:0.25 delay:0.0
			options:UIViewAnimationOptionCurveLinear | UIViewAnimationOptionAllowUserInteraction
			animations:^(void)
			{
				self.alpha = 0.0f;
			}
			completion:^(BOOL finished)
			{
				self.hidden = YES;
			}
		];
	}
}

- (void)showToolbar
{
	if (self.hidden == YES)
	{
		[UIView animateWithDuration:0.25 delay:0.0
			options:UIViewAnimationOptionCurveLinear | UIViewAnimationOptionAllowUserInteraction
			animations:^(void)
			{
				self.hidden = NO;
				self.alpha = 1.0f;
			}
			completion:NULL
		];
	}
}

#pragma mark UIButton action methods

- (void)closeButtonTapped:(UIButton *)button {
	[delegate tappedInToolbar:self closeButton:button];
}

- (void)syncButtonTapped:(UIButton *)button {
	[delegate tappedInToolbar:self syncButton:button];
}

- (void)pointButtonTapped:(UIButton *)button {
	[delegate tappedInToolbar:self pointButton:button];
}

- (void)privateEditButtonTapped:(UIButton *)button {
	[delegate tappedInToolbar:self privateEditButton:button];
}

- (void)publicEditButtonTapped:(UIButton *)button {
	[delegate tappedInToolbar:self publicEditButton:button];
}

- (void)publishButtonTapped:(UIButton *)button {
	[delegate tappedInToolbar:self publishButton:button];
}

@end
