//
//  Bundle+Additions.m
//  Partiture
//
//  Created by Kaarel Kruus on 16/03/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "Bundle+Additions.h"
#import "PRTAppDelegate.h"
#import "Score+Additions.h"
#import "Member+Additions.h"
#import "Membership+Additions.h"
#import "Membership+Enums.h"

@implementation Bundle (Additions)

+ (Bundle *)bundleWithAbstractScore:(Score *)score forUser:(User *)user {
    if(user) {
        
        // create the bundle object first // TODO: what if bundle exists?
        Bundle *newBundle = [NSEntityDescription insertNewObjectForEntityForName:NSStringFromClass(Bundle.class)
                                                          inManagedObjectContext:[APPDELEGATE managedObjectContext]];
        
        // ensure the managed score will be created
        Score *managedScore = [Score ensureManagedScoreWithScore:score];
        
        // bind
        newBundle.score = managedScore;
        newBundle.user = user;
        
        [APPDELEGATE saveContext];
        return newBundle;
        
    } else {
        // TODO: error management
        return nil;
    }
}

- (void)loadOwnedRooms:(NSArray **)ownedRooms andGuestRooms:(NSArray **)guestRooms {
    
    User *bundleCreator = self.user;
    
    NSMutableArray *owner = [NSMutableArray array];
    NSMutableArray *guest = [NSMutableArray array];
    
    for (Room *room in self.rooms) {
        Member *member = [Member fetchMemberWithToken:bundleCreator.token];
        if(member) {
            Membership *memberShip = [Membership fetchMembershipForMember:member inRoom:room];
            if([memberShip.role isEqual:@(MembershipRoleRoomOwner)]) {
                [owner addObject:room];
            } else if([memberShip.role isEqual:@(MembershipRoleRoomGuest)]) {
                [guest addObject:room];
            } else {
                // TODO: unexpected error
            }
        } else {
            // TODO: unexpected error
        }
    }
    
    *ownedRooms = owner;
    *guestRooms = guest;
}

@end
