//
//  DAOManager.m
//  Partiture
//
//  Created by Kaarel Kruus on 15/03/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "DAOManager.h"
#import "TBXML+HTTP.h"

#import "Score+Additions.h"

@implementation DAOManager

static DAOManager *sharedManager = nil;

+ (id)sharedManager {
    static dispatch_once_t token;
    dispatch_once(&token, ^{
        sharedManager = [[DAOManager alloc] init];
    });
    return sharedManager;
}

- (void)loadAvailableAbstractScoreListForUser:(User *)user withCompletionBlock:(PRTResultBlock)completionBlock {
    
    NSMutableArray *availableScores = [NSMutableArray array];
    
    NSString *listUrlString = [NSString stringWithFormat:SERVICE_CONFIGURATION[@"ListUrl"], user.token];
    NSURL *url = [NSURL URLWithString:listUrlString];
    
    if(url) {
        
        TBXML *tbxml = nil;
        
        tbxml = [[TBXML alloc] initWithURL:url success:^(TBXML *tbxml) {
            
            TBXMLElement *root = tbxml.rootXMLElement;
            
            TBXMLElement *entry = [TBXML childElementNamed:@"fileset" parentElement:root];
            
            while (entry != nil) {
                
                Score *score = [Score unassociatedObject];
                score.title = [TBXML valueOfAttributeNamed:@"title" forElement:entry];
                score.token = [TBXML valueOfAttributeNamed:@"checksum" forElement:entry];
                //score.href = [TBXML valueOfAttributeNamed:@"href" forElement:entry];
                //score.origin = [TBXML valueOfAttributeNamed:@"origin" forElement:entry];
                //aScore.composers // TODO
                //aScore.lyricists // TODO
                //aScore.arrangers // TODO
                [availableScores addObject:score];
                
                entry = [TBXML nextSiblingNamed:@"fileset" searchFromElement:entry];
            }
            
            if(completionBlock) {
                completionBlock(availableScores, nil);
            }
            
        } failure:^(TBXML *tbxml, NSError *error) {
            
            if(completionBlock) {
                completionBlock(nil, error); // TODO: error management
            }
            
        }];
        
    } else {
        // TODO: error management
        completionBlock(nil, nil);
    }
}

@end
