//
//  PRTAppDelegate.m
//  Partiture
//
//  Created by Kaarel Kruus on 08/02/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "PRTAppDelegate.h"
#import "PRTCommunicationManager.h"
#import <MBProgressHUD/MBProgressHUD.h>
#import "User+Additions.h"

NSString *PRTApplicationDidBecomeActiveNotification = @"PRTApplicationDidBecomeActiveNotification";

@interface PRTAppDelegate ()

@property (nonatomic, strong) NSNumber *isListening;
@property (readwrite, strong, nonatomic) User *appUser;

@end

@implementation PRTAppDelegate (ListenerAccessories)

- (void)startListening {
    @synchronized (self.isListening) {
        if(!self.isListening.boolValue) {
            self.isListening = @(YES);
            if(self.appUser) {
                [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:YES];
                [COM_LISTENER startDiscoveringRoomSessionsWithUserToken:self.appUser.token];
            } else {
                // TODO: handle
            }
        } else {
            // TODO: handle
        }
    }
}

- (void)stopListening {
    @synchronized (self.isListening) {
        if(self.isListening.boolValue) {
            self.isListening = @(NO);
            if(self.appUser) {
                [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:NO];
                [COM_LISTENER stopDiscoveringRoomSessions];
            } else {
                // TODO: handle
            }
        } else {
            // TODO: handle
        }
    }
}

@end

@implementation PRTAppDelegate (DebugAccessories)

- (void)debugHudWithText:(NSString *)text {
    [self debugHudWithTextFormat:text];
}

- (void)debugHudWithTextFormat:(NSString *)formatString, ... {
    
    NSString *string;
    
    va_list va_args;
    va_start(va_args, formatString);
    string = [[NSString alloc] initWithFormat:formatString arguments:va_args];
    va_end(va_args);
    
    // TODO: remove for release
    DISPATCH_ASYNC_MAIN({
        MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[UIApplication sharedApplication].keyWindow
                                                  animated:YES];
        hud.labelText = string;
        [hud hide:YES afterDelay:2.0F];
    });
}

@end

@implementation PRTAppDelegate

@synthesize managedObjectContext = _managedObjectContext;
@synthesize managedObjectModel = _managedObjectModel;
@synthesize persistentStoreCoordinator = _persistentStoreCoordinator;

- (void)setIsEditingMode:(BOOL)isEditingMode {
    self->_isEditingMode = isEditingMode;
    [[NSNotificationCenter defaultCenter] postNotificationName:@"EditingModeChanged" object:nil];
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    
    User *activeUser = [User fetchActiveUser];
    
    if(activeUser) {
        NSLog(@"ACTIVE USER FOUND!");
        self.appUser = activeUser;
    } else {
        NSLog(@"NO ACTIVE USER!");
        User *user = [User userWithToken:@"14d678e1-1deb-4f45-8a32-681d6cbd28f1" andName:@"John Paige"];
        //User *user = [User userWithToken:@"e48ffd92-d425-423c-bfba-a4dcd80ba98f" andName:@"Jack Green"];
        user.isActive = YES;
        [self saveContext];
        
        self.appUser = user;
        
    }
    
    NSLog(@"Started app with user: %@ (%@)", self.appUser.name, self.appUser.token);
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
    //NSLog(@"%s", __PRETTY_FUNCTION__);
    [self stopListening];
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
    //NSLog(@"%s", __PRETTY_FUNCTION__);
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
    //NSLog(@"%s", __PRETTY_FUNCTION__);
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
    //NSLog(@"%s", __PRETTY_FUNCTION__);
    
    // always start listening (advertising) here already
    [self startListening];
    
    // notfiy other application components as well
    [[NSNotificationCenter defaultCenter] postNotificationName:PRTApplicationDidBecomeActiveNotification
                                                        object:nil
                                                      userInfo:nil];
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Saves changes in the application's managed object context before the application terminates.
    [self saveContext];
}

- (void)saveContext
{
    NSError *error = nil;
    NSManagedObjectContext *managedObjectContext = self.managedObjectContext;
    if (managedObjectContext != nil) {
        if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error]) {
             // Replace this implementation with code to handle the error appropriately.
             // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. 
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            abort();
        } 
    }
}

#pragma mark - Core Data stack

// Returns the managed object context for the application.
// If the context doesn't already exist, it is created and bound to the persistent store coordinator for the application.
- (NSManagedObjectContext *)managedObjectContext
{
    if (_managedObjectContext != nil) {
        return _managedObjectContext;
    }
    
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil) {
        _managedObjectContext = [[NSManagedObjectContext alloc] init];
        [_managedObjectContext setPersistentStoreCoordinator:coordinator];
    }
    return _managedObjectContext;
}

// Returns the managed object model for the application.
// If the model doesn't already exist, it is created from the application's model.
- (NSManagedObjectModel *)managedObjectModel
{
    if (_managedObjectModel != nil) {
        return _managedObjectModel;
    }
    NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"Partiture" withExtension:@"momd"];
    _managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    return _managedObjectModel;
}

// Returns the persistent store coordinator for the application.
// If the coordinator doesn't already exist, it is created and the application's store added to it.
- (NSPersistentStoreCoordinator *)persistentStoreCoordinator
{
    if (_persistentStoreCoordinator != nil) {
        return _persistentStoreCoordinator;
    }
    
    NSURL *storeURL = [[self applicationDocumentsDirectory] URLByAppendingPathComponent:@"Partiture.sqlite"];
    
    NSError *error = nil;
    _persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[self managedObjectModel]];
    if (![_persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:storeURL options:nil error:&error]) {
        /*
         Replace this implementation with code to handle the error appropriately.
         
         abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. 
         
         Typical reasons for an error here include:
         * The persistent store is not accessible;
         * The schema for the persistent store is incompatible with current managed object model.
         Check the error message to determine what the actual problem was.
         
         
         If the persistent store is not accessible, there is typically something wrong with the file path. Often, a file URL is pointing into the application's resources directory instead of a writeable directory.
         
         If you encounter schema incompatibility errors during development, you can reduce their frequency by:
         * Simply deleting the existing store:
         [[NSFileManager defaultManager] removeItemAtURL:storeURL error:nil]
         
         * Performing automatic lightweight migration by passing the following dictionary as the options parameter:
         @{NSMigratePersistentStoresAutomaticallyOption:@YES, NSInferMappingModelAutomaticallyOption:@YES}
         
         Lightweight migration will only work for a limited set of schema changes; consult "Core Data Model Versioning and Data Migration Programming Guide" for details.
         
         */
        NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        abort();
    }    
    
    return _persistentStoreCoordinator;
}

#pragma mark - Application's Documents directory

// Returns the URL to the application's Documents directory.
- (NSURL *)applicationDocumentsDirectory
{
    return [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject];
}

@end
