//
//  PRTCOMListener.m
//  Partiture
//
//  Created by Kaarel Kruus on 17/03/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "PRTCOMListener.h"
#import "PRTAppDelegate.h"
#import "PRTCommunication.h"
#import "PRTCommunicator+Private.h"
#import "PRTInvitationInfo.h"
#import "PRTPeer.h"

NSString *PRTCOMListenerRoomSessionsUpdateNotification = @"PRTCOMListenerRoomSessionsUpdateNotification";


@interface PRTCOMListener (Interal)

@end

@interface PRTCOMListener () <MCNearbyServiceAdvertiserDelegate, MCSessionDelegate>

@property (nonatomic, strong) MCPeerID *peerID;
//@property (nonatomic, strong) MCSession *session;
@property (nonatomic, strong) NSMutableArray *connectionSessions;
@property (nonatomic, strong) MCNearbyServiceAdvertiser *serviceAdvertiser;

/*
@property (nonatomic, strong) NSMutableArray *connectedPeers;
@property (nonatomic, strong) NSMutableArray *connectingPeers;
@property (nonatomic, strong) NSMutableArray *notConnectedPeers;
*/

@property (nonatomic, strong) NSMutableDictionary *peerDictionary;

@property (nonatomic, strong) NSNumber *isAdvertising;

@end

@implementation PRTCOMListener

- (id)init {
    self = [super init];
    if (self) {
        
        self.peerDictionary = [[NSMutableDictionary alloc] init];
        
        self.connectionSessions = [[NSMutableArray alloc] init];
        /*
        self.connectedPeers = [[NSMutableArray alloc] init];
        self.connectingPeers = [[NSMutableArray alloc] init];
        self.notConnectedPeers = [[NSMutableArray alloc] init];
        */
    }
    return self;
}

// Remote peer changed state
- (void)session:(MCSession *)session peer:(MCPeerID *)peerID didChangeState:(MCSessionState)state {
    PRTCOMListener_LOG(@"%s %li", __PRETTY_FUNCTION__, state);
    
    if(state == MCSessionStateConnected) {
        /*
        [self.connectedPeers addObject:peerID];
        [self.notConnectedPeers removeObject:peerID];
        [self.connectingPeers removeObject:peerID];
        */
    } else if(state == MCSessionStateNotConnected) {
        /*
        [self.notConnectedPeers addObject:peerID];
        [self.connectedPeers removeObject:peerID];
        [self.connectingPeers removeObject:peerID];
        */
    } else if(state == MCSessionStateConnecting) {
        /*
        [self.connectingPeers addObject:peerID];
        [self.connectedPeers removeObject:peerID];
        [self.notConnectedPeers removeObject:peerID];
        */
    }
    
    @synchronized (self.peerDictionary) {
        
        if(![self.peerDictionary objectForKey:peerID]) {
            
            // this is a new peer, let us wait for more info
            PRTPeer *peer = [[PRTPeer alloc] init];
            peer.info = nil;
            peer.visible = NO;
            
            [self.peerDictionary setObject:peer forKey:peerID];
            
        } else {
            
            
            
            // the connection state of an registered peer changed (maybe dropped, maybe reconnected?)
            [[NSNotificationCenter defaultCenter] postNotificationName:PRTCOMListenerRoomSessionsUpdateNotification object:nil userInfo:nil];
        }
        
    }
}

// Received data from remote peer
- (void)session:(MCSession *)session didReceiveData:(NSData *)data fromPeer:(MCPeerID *)peerID {
    PRTCOMListener_LOG(@"%s", __PRETTY_FUNCTION__);
    
    [self uncompressDataPackage:data withCompletion:^(NSString *uuid, id body, id responseBlock) {
        
        if([body isKindOfClass:[PRTRoomInfo class]]) {
            
            
            
        } else if([body isKindOfClass:[PRTRoomJoinResponseInfo class]]) {
            
            PRTRoomJoinResponseInfo *responseInfo = (PRTRoomJoinResponseInfo *)body;
            __block PRTCOMRoomJoinResponseBlock _responseBlock = responseBlock;
            if(_responseBlock) {
                _responseBlock(responseInfo.accepted);
            }
            
        } else if([body isKindOfClass:[PRTPageTurnInfo class]]) {
            
            PRTPageTurnInfo *pageTurnInfo = (PRTPageTurnInfo *)body;
            PRT_LOG_S(@"Turn page: %i", pageTurnInfo.pageIndex);
            
        }
    }];
}

// Received a byte stream from remote peer
- (void)session:(MCSession *)session didReceiveStream:(NSInputStream *)stream withName:(NSString *)streamName fromPeer:(MCPeerID *)peerID {
    //PRTCOMListener_LOG(@"%s", __PRETTY_FUNCTION__);
}

// Start receiving a resource from remote peer
- (void)session:(MCSession *)session didStartReceivingResourceWithName:(NSString *)resourceName fromPeer:(MCPeerID *)peerID withProgress:(NSProgress *)progress {
    //PRTCOMListener_LOG(@"%s", __PRETTY_FUNCTION__);
}

// Finished receiving a resource from remote peer and saved the content in a temporary location - the app is responsible for moving the file to a permanent location within its sandbox
- (void)session:(MCSession *)session didFinishReceivingResourceWithName:(NSString *)resourceName fromPeer:(MCPeerID *)peerID atURL:(NSURL *)localURL withError:(NSError *)error {
    //PRTCOMListener_LOG(@"%s", __PRETTY_FUNCTION__);
}

// Made first contact with peer and have identity information about the remote peer (certificate may be nil)
- (void)session:(MCSession *)session didReceiveCertificate:(NSArray *)certificate fromPeer:(MCPeerID *)peerID certificateHandler:(void(^)(BOOL accept))certificateHandler {
    //PRTCOMListener_LOG(@"%s", __PRETTY_FUNCTION__);
    if(certificateHandler) { // MCF known issue workaround
        certificateHandler(YES);
    }
}

// Incoming invitation request.  Call the invitationHandler block with YES and a valid session to connect the inviting peer to the session.
- (void)advertiser:(MCNearbyServiceAdvertiser *)advertiser didReceiveInvitationFromPeer:(MCPeerID *)peerID withContext:(NSData *)context invitationHandler:(void(^)(BOOL accept, MCSession *session))invitationHandler {
    PRTCOMListener_LOG(@"%s", __PRETTY_FUNCTION__);
    
    [self uncompressDataPackage:context withCompletion:^(NSString *uuid, id body, id responseBlock) {
        
        if([body isKindOfClass:[PRTInvitationInfo class]]) {
        
            MCSession *connectionSession = [self nextSession];
            
            PRTInvitationInfo *info = (PRTInvitationInfo *)body;
            if(info.invitationStatus == InvitationStatusHello) {
                
                // we expect here also a linked item with the room info
                if([info.linkedInfo isKindOfClass:[PRTRoomInfo class]]) {

                    PRTRoomInfo *roomInfo = (PRTRoomInfo *)info.linkedInfo;
                    roomInfo.roomName = peerID.displayName;
                    roomInfo.bridge = connectionSession;
                    
                    @synchronized (self.peerDictionary) {
                        
                        PRTPeer *peer = [self.peerDictionary objectForKey:peerID];
                        if(!peer) {
                            peer = [[PRTPeer alloc] init];
                        }
                        peer.info = roomInfo;
                        peer.visible = YES;
                        [self.peerDictionary setObject:peer forKey:peerID];
                    }
                    
                    [[NSNotificationCenter defaultCenter] postNotificationName:PRTCOMListenerRoomSessionsUpdateNotification
                                                                        object:nil userInfo:nil];
                    
                }
            }
            
            // automatically accept the invitation to establish the initial connection
            invitationHandler(YES, connectionSession);
        }
    }];
    
    

}

// since the messages can be sent to sessions and not to individual peers, create a new session for each peer
- (MCSession *)nextSession {
    MCSession *session = [[MCSession alloc] initWithPeer:self.peerID];
    session.delegate = self;
    [self.connectionSessions addObject:session];
    return session;
}

// Advertising did not start due to an error
- (void)advertiser:(MCNearbyServiceAdvertiser *)advertiser didNotStartAdvertisingPeer:(NSError *)error {
    //PRTCOMListener_LOG(@"%s", __PRETTY_FUNCTION__);
    // TODO: try again?
}

@end

@implementation PRTCOMListener (Interal)

@end

@implementation PRTCOMListener (External)

- (void)startDiscoveringRoomSessionsWithUserToken:(NSString *)userToken {
    PRTCOMListener_LOG(@"%s (%@)", __PRETTY_FUNCTION__, userToken);
    
    @synchronized (self.isAdvertising) {
        if(!self.isAdvertising.boolValue) {
            self.isAdvertising = @(YES);
            
            self.peerID = [[MCPeerID alloc] initWithDisplayName:userToken];
            self.serviceAdvertiser = [[MCNearbyServiceAdvertiser alloc] initWithPeer:self.peerID
                                                                       discoveryInfo:nil
                                                                         serviceType:INFO_DICTIONARY[@"BonjourServiceType"]];
            self.serviceAdvertiser.delegate = self;
            
            // advertise oneself, so that the inital connection could be built up between the peers
            [self.serviceAdvertiser startAdvertisingPeer];
            
        } else {
            // TODO: handle
        }
    }
}

- (void)stopDiscoveringRoomSessions {
    PRTCOMListener_LOG(@"%s", __PRETTY_FUNCTION__);
    
    @synchronized (self.isAdvertising) {
        if(self.isAdvertising.boolValue) {
            self.isAdvertising = @(NO);
            
            [self.serviceAdvertiser stopAdvertisingPeer];
            self.serviceAdvertiser = nil;
            self.peerID = nil;
            
        } else {
            // TODO: handle
        }
    }
}

- (NSArray *)visibleRooms {
    NSMutableArray *result = [NSMutableArray array];
    for (PRTPeer *peer in self.peerDictionary.allValues) {
        if(peer.info != nil && peer.visible == YES) {
            [result addObject:peer.info];
        }
    }
    return result;
}

- (void)pushRoomJoinRequest:(PRTRoomJoinRequestInfo *)requestInfo inRoom:(PRTRoomInfo *)roomInfo {
    PRT_LOG_S(@"Push join request");
    [self pushPackage:requestInfo inSession:roomInfo.bridge withResponseBlock:requestInfo.responseBlock];
}

@end
