//
//  PRTCOMPublisher.m
//  Partiture
//
//  Created by Kaarel Kruus on 17/03/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "PRTCOMPublisher.h"
#import "PRTCommunicator+Private.h"
#import "PRTCommunication.h"
#import <MultipeerConnectivity/MultipeerConnectivity.h>
#import "PRTAppDelegate.h"
#import "PRTRoomInfo.h"
#import "PRTInvitationInfo.h"
#import "Membership+Enums.h"

@interface PRTPeerSessionContainer : NSObject

@property (nonatomic, strong) MCSession *session;
@property (nonatomic, assign) BOOL isReadyToReceiveData;

+ (PRTPeerSessionContainer *)peerSessionContainerWithSession:(MCSession *)session;
+ (PRTPeerSessionContainer *)peerSessionContainerWithSession:(MCSession *)session andState:(BOOL)isReadyToReceiveData;

@end

@implementation PRTPeerSessionContainer

+ (PRTPeerSessionContainer *)peerSessionContainerWithSession:(MCSession *)session {
    return [self peerSessionContainerWithSession:session andState:NO];
}

+ (PRTPeerSessionContainer *)peerSessionContainerWithSession:(MCSession *)session andState:(BOOL)isReadyToReceiveData {
    return [[self alloc] initWithSession:session andState:isReadyToReceiveData];
}

- (id)initWithSession:(MCSession *)session andState:(BOOL)isReadyToReceiveData {
    self = [super init];
    if (self) {
        self.session = session;
        self.isReadyToReceiveData = isReadyToReceiveData;
    }
    return self;
}

@end

@interface PRTCOMPublisher () <MCNearbyServiceBrowserDelegate, MCSessionDelegate>

@property (nonatomic, strong) Session *currentSession;

@property (nonatomic, strong) MCPeerID *peerID;
@property (nonatomic, strong) NSMutableDictionary *peerSessionContainers;

@property (nonatomic, strong) MCNearbyServiceBrowser *serviceBrowser;

//@property (nonatomic, strong) NSMutableArray *foundPeers;

/*
@property (nonatomic, strong) NSMutableArray *connectedPeers;
@property (nonatomic, strong) NSMutableArray *connectingPeers;
@property (nonatomic, strong) NSMutableArray *notConnectedPeers;
*/

//@property (nonatomic, strong) NSMutableDictionary *pendingInvitees;
//@property (nonatomic, strong) NSTimer *timeoutTimer;
//@property (nonatomic, strong) NSNumber *isBrowsing;

@end

@implementation PRTCOMPublisher

- (void)addPeerSessionContainer:(PRTPeerSessionContainer *)container withKey:(NSString *)key {
    @synchronized (self.peerSessionContainers) {
        [self.peerSessionContainers setObject:container forKey:key];
    }
}

- (void)removePeerSessionContainerWithKey:(NSString *)key {
    @synchronized (self.peerSessionContainers) {
        [self.peerSessionContainers removeObjectForKey:key];
    }
}

- (NSArray *)receivingSessions {
    // TODO: optimize
    NSMutableArray *result = [NSMutableArray array];
    for (PRTPeerSessionContainer *container in self.peerSessionContainers.allValues) {
        if(container.isReadyToReceiveData) {
            [result addObject:container.session];
        }
    }
    return result;
}

// since the messages can be sent to sessions and not to individual peers, create a new session for each peer
- (MCSession *)nextSessionWithRemotePeer:(MCPeerID *)peerID {
    MCSession *session = [[MCSession alloc] initWithPeer:self.peerID
                                        securityIdentity:nil
                                    encryptionPreference:MCEncryptionRequired]; // ENHANCCE: change encryption if needed during the runtime
    session.delegate = self;
    [self addPeerSessionContainer:[PRTPeerSessionContainer peerSessionContainerWithSession:session]
                          withKey:peerID.displayName];
    return session;
}

- (id)init {
    self = [super init];
    if (self) {
        
        //self.isBrowsing = @(NO);
        
        //self.foundPeers = [[NSMutableArray alloc] init];
        
        self.peerSessionContainers = [[NSMutableDictionary alloc] init];
        
        /*
        self.connectedPeers = [[NSMutableArray alloc] init];
        self.connectingPeers = [[NSMutableArray alloc] init];
        self.notConnectedPeers = [[NSMutableArray alloc] init];
        */
        
        //self.pendingInvitees = [[NSMutableDictionary alloc] init];
        
    }
    return self;
}

// Remote peer changed state
- (void)session:(MCSession *)session peer:(MCPeerID *)peerID didChangeState:(MCSessionState)state {
    PRTCOMPublisher_LOG(@"%s %li", __PRETTY_FUNCTION__, state);
    
    if(state == MCSessionStateConnected) {
        
        /*
        [self.connectedPeers addObject:peerID];
        [self.notConnectedPeers removeObject:peerID];
        [self.connectingPeers removeObject:peerID];
        */
        
        // send more information to the connected peer(s) // TODO: always?
        /*
        PRTRoomInfo *roomInfo = [PRTRoomInfo infoWithRoomUuid:self.currentSession.room.uuid
                                                andScoreToken:self.currentSession.room.bundle.score.token];
        [self pushPackage:roomInfo inSession:session];
        */
        
    } else if(state == MCSessionStateNotConnected) {
        
        /*
        [self.notConnectedPeers addObject:peerID];
        [self.connectedPeers removeObject:peerID];
        [self.connectingPeers removeObject:peerID];
        */
        
        //[self connectionLostWithPeer:peerID];
        
    } else if(state == MCSessionStateConnecting) {
        
        /*
        [self.connectingPeers addObject:peerID];
        [self.connectedPeers removeObject:peerID];
        [self.notConnectedPeers removeObject:peerID];
        */
        
    }
}

/*
- (void)connectionLostWithPeer:(MCPeerID *)peerID {
    //NSLog(@"%s", __PRETTY_FUNCTION__);
    // a peer was disconnected (random connection drop, app crash, just left the room... -
    // check if we should re-establish the connection
    if(self.delegate) {
        
        //BOOL shouldReconnect = [self.delegate publisher:self shouldReconnectMemberWithUuid:peerID.displayName inSession:self.currentSession];
    
        BOOL shouldReconnect = YES; // TOOD: remove
        if (shouldReconnect) {
            NSLog(@"ReConnect!");
            [self invitePeer:peerID toSession:self.session]; // invite again
        } else {
            //TODO: handle
        }
    } else {
        //TODO: handle
    }
}
*/

// Received data from remote peer
- (void)session:(MCSession *)session didReceiveData:(NSData *)data fromPeer:(MCPeerID *)peerID {
    PRTCOMPublisher_LOG(@"%s", __PRETTY_FUNCTION__);
    
    [self uncompressDataPackage:data withCompletion:^(NSString *uuid, id body, id responseBlock) {
    
        /*
         RECEIVE JOIN REQUEST
         */
        if([body isKindOfClass:[PRTRoomJoinRequestInfo class]]) {
            PRTRoomJoinRequestInfo *requestInfo = (PRTRoomJoinRequestInfo *)body;
            [self.currentSession receiveJoinRequestFromUserWithName:requestInfo.userName
                                                              token:requestInfo.userToken
                                                         andPasskey:requestInfo.passkey
                                                  withDecisionBlock:^(BOOL accept) {
                                                      
                                                      if(accept) { // TODO: better check here
                                                          @synchronized (self.peerSessionContainers) {
                                                              PRTPeerSessionContainer *container = self.peerSessionContainers[peerID.displayName];
                                                              if(container) {
                                                                  container.isReadyToReceiveData = YES;
                                                              } else {
                                                                  // TODO: handle
                                                              }
                                                          }
                                                      } else {
                                                          // TODO: handle
                                                      }
                                                      
                                                      /*
                                                       PUSH JOIN REQUEST RESPONSE
                                                       */
                                                      PRTRoomJoinResponseInfo *info = [PRTRoomJoinResponseInfo infoWithResponse:accept];
                                                      [self pushPackage:info withUuid:uuid inSession:session];
                                                      
                                                  }];
        }
    }];
}

- (void)browser:(MCNearbyServiceBrowser *)browser foundPeer:(MCPeerID *)peerID withDiscoveryInfo:(NSDictionary *)info {
    PRTCOMPublisher_LOG(@"%s", __PRETTY_FUNCTION__);
    
    // ignore itself // TODO: better check here
    if(![[APPDELEGATE appUser].token isEqualToString:peerID.displayName]) {
        
        if(self.delegate) {
            
            // check the possible membership for the received token
            Membership *membership = [self.delegate publisher:self
                                 membershipForMemberWithToken:peerID.displayName
                                                       inRoom:self.currentSession.room];
            
            // maybe was even participating in the current session?
            BOOL isParticipant = [self.delegate publisher:self
                                   hasParticipatingMember:membership.member
                                                inSession:self.currentSession];
            
            // TODO: move this logic higher
            PRTInvitationInfo *invitationInfo;
            
            if(isParticipant) {
                
                invitationInfo = [PRTInvitationInfo infoWithInvitationStatus:InvitationStatusWelcomeBack];
                NSLog(@"WelcomeBack!"); // TODO: what if the member has re-installed / cleaned / has not the info?
                
            } else if([membership.status isEqual:@(MembershipStatusAccepted)]) {
                
                invitationInfo = [PRTInvitationInfo infoWithInvitationStatus:InvitationStatusWelcome];
                NSLog(@"Welcome!");
                
            } else if([membership.status isEqual:@(MembershipStatusAcceptedPermanently)]) {
                
                invitationInfo = [PRTInvitationInfo infoWithInvitationStatus:InvitationStatusWarmWelcome];
                NSLog(@"WarmWelcome!");
                
            } else if([membership.status isEqual:@(MembershipStatusBlocked)]) {
                
                invitationInfo = [PRTInvitationInfo infoWithInvitationStatus:InvitationStatusGoodBye];
                NSLog(@"GoodBye!");
                
            } else {
                
                // since we see this user for the first time, push also the RoomInfo as a linked item
                PRTRoomInfo *roomInfo = [PRTRoomInfo infoWithRoomUuid:self.currentSession.room.uuid
                                                        andScoreToken:self.currentSession.room.bundle.score.token];
                invitationInfo = [PRTInvitationInfo infoWithInvitationStatus:InvitationStatusHello andLinkedInfo:roomInfo];
                NSLog(@"Hello!");
                
            }
            
            NSData *data = [self compressDataPackageWith:invitationInfo];
            [self invitePeer:peerID withContext:data toSession:[self nextSessionWithRemotePeer:peerID]];
            
        } else {
            // TODO: handle
        }
        
    } else {
        // TODO: handle
    }
}

- (void)invitePeer:(MCPeerID *)peerID withContext:(NSData *)data toSession:(MCSession *)session {
    NSLog(@"%s (UUID: %@)", __PRETTY_FUNCTION__, peerID.displayName);
    // TODO: timeout? send again after timeout?
    // TODO: what if the accept does not come?
    [self.serviceBrowser invitePeer:peerID toSession:session withContext:data timeout:0.0F];
}

/*
- (void)addPendingInvitee:(MCPeerID *)invitee atTime:(NSDate *)date {
    @synchronized (self.pendingInvitees) {
        
        // add to pending invitees - so that after the timeout the invitation could be resent to the invitee
        [self.pendingInvitees setObject:[NSDate date] forKey:invitee];
        
        // if we add someting to the dictionary, also start the timer for tracking the timeouts
        if(!self.timeoutTimer) {
            self.timeoutTimer = [NSTimer scheduledTimerWithTimeInterval:0.5F target:self selector:@selector(onTick:) userInfo:nil repeats:YES]; // TODO: define interval
        }
    }
}
*/
/*
- (void)onTick:(NSTimer *)timer {
    NSLog(@"TICK!");
}
*/

- (void)browser:(MCNearbyServiceBrowser *)browser lostPeer:(MCPeerID *)peerID {
    PRTCOMPublisher_LOG(@"%s", __PRETTY_FUNCTION__);
    @synchronized (self.peerSessionContainers) {
        [self.peerSessionContainers removeObjectForKey:peerID.displayName];
    }
}

- (void)browser:(MCNearbyServiceBrowser *)browser didNotStartBrowsingForPeers:(NSError *)error {
    //TODO: handle
    PRTCOMPublisher_LOG(@"%s", __PRETTY_FUNCTION__);
}

// Received a byte stream from remote peer
- (void)session:(MCSession *)session didReceiveStream:(NSInputStream *)stream withName:(NSString *)streamName fromPeer:(MCPeerID *)peerID { }

// Start receiving a resource from remote peer
- (void)session:(MCSession *)session didStartReceivingResourceWithName:(NSString *)resourceName fromPeer:(MCPeerID *)peerID withProgress:(NSProgress *)progress { }

// Finished receiving a resource from remote peer and saved the content in a temporary location - the app is responsible for moving the file to a permanent location within its sandbox
- (void)session:(MCSession *)session didFinishReceivingResourceWithName:(NSString *)resourceName fromPeer:(MCPeerID *)peerID atURL:(NSURL *)localURL withError:(NSError *)error { }

// Made first contact with peer and have identity information about the remote peer (certificate may be nil)
- (void)session:(MCSession *)session didReceiveCertificate:(NSArray *)certificate fromPeer:(MCPeerID *)peerID certificateHandler:(void(^)(BOOL accept))certificateHandler {
    //PRTCOMPublisher_LOG(@"%s", __PRETTY_FUNCTION__);
    if(certificateHandler) { // MCF known issue workaround
        certificateHandler(YES);
    }
}

@end

@implementation PRTCOMPublisher (External)

- (void)publishSession:(Session *)session {
    PRTCOMPublisher_LOG(@"%s (%@)", __PRETTY_FUNCTION__, session.uuid);
    /*
    @synchronized (self.isBrowsing) {
        if(!self.isBrowsing.boolValue) {
            self.isBrowsing = @(YES);
        } else {
            // TODO: handle
        }
    }
    */
    //PRTCOMPublisher_LOG(@"%s", __PRETTY_FUNCTION__);
    
    // TODO: what to do with the current/active session - unpublish?
    self.currentSession = session;
    
    // start looking for peers to build up a initial connection
    self.peerID = [[MCPeerID alloc] initWithDisplayName:self.currentSession.room.name]; // TODO: what to use for display name? check max limit (63 bytes in UTF-8)!
    self.serviceBrowser = [[MCNearbyServiceBrowser alloc] initWithPeer:self.peerID
                                                           serviceType:INFO_DICTIONARY[@"BonjourServiceType"]];
    self.serviceBrowser.delegate = self;
    [self.serviceBrowser startBrowsingForPeers]; // TODO: pause browsing if all awaited peers are here?
}

- (void)unpublishSession {
    PRTCOMPublisher_LOG(@"%s (%@)", __PRETTY_FUNCTION__, self.currentSession.uuid);
    /*
    @synchronized (self.isBrowsing) {
        if(self.isBrowsing.boolValue) {
            self.isBrowsing = @(NO);
        } else {
            // TODO: handle
        }
    }
     */
    [self.serviceBrowser stopBrowsingForPeers];
}

- (void)pushPageTurn:(PRTPageTurnInfo *)pageTurnInfo {
    [self pushPackage:pageTurnInfo inSessions:self.receivingSessions];
}

@end
