//
//  PRTCommunicator.m
//  Partiture
//
//  Created by Kaarel Kruus on 09/04/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "PRTCommunicator.h"
#import "PRTCommunicator+Private.h"

@interface PRTCommunicator ()

@property (nonatomic, strong) NSMutableDictionary *responseBlockDictionary;

@end

@implementation PRTCommunicator

- (NSData *)compressDataPackageWith:(id<NSCoding>)packageBody {
    return [self compressDataPackageWith:packageBody andUuid:nil];
}

- (NSData *)compressDataPackageWith:(id<NSCoding>)packageBody andUuid:(NSString *)uuid {
    
    // when no UUID provided, create one
    uuid = (!uuid || uuid.length == 0) ? [PRTUtils generateUUID] : uuid;
    
    // compose header & body
    NSDictionary *dictionary = @{ @"header": @{ @"id": uuid }, @"body": packageBody };
    
    // archive
    return [NSKeyedArchiver archivedDataWithRootObject:dictionary];
}

- (void)uncompressDataPackage:(NSData *)dataPackage withCompletion:(PRTCOMPackageExtractBlock)extractBlock {
    
    // unarchive
    NSDictionary *package = (NSDictionary *)[NSKeyedUnarchiver unarchiveObjectWithData:dataPackage];
    
    NSString *uuid = package[@"header"][@"id"];
    id responseBlock = self.responseBlockDictionary[uuid];
    
    if(extractBlock) {
        extractBlock(uuid, package[@"body"], responseBlock);
    }
}

- (void)pushPackage:(id)package withUuid:(NSString *)uuid inSessions:(NSArray *)sessions withResponseBlock:(id)resposeBlock {
    
    // send to all targeted sessions
    // TODO: ensure that the messages are sent out the same time
    for (MCSession *session in sessions) {
        
        // each pushed package should have a unique id for tracking
        uuid = (!uuid || uuid.length == 0) ? [PRTUtils generateUUID] : uuid;
        
        // save block for submitting the response
        if(resposeBlock) {
            [self.responseBlockDictionary setObject:resposeBlock forKey:uuid];
        }
        
        NSData *data = [self compressDataPackageWith:package andUuid:uuid];
        
        NSError *error;
        
        BOOL send = [session sendData:data
                              toPeers:session.connectedPeers
                             withMode:MCSessionSendDataReliable
                                error:&error]; // TODO: config transmission mode
        if(!send || error) {
            // TODO: what here? try again?
            PRT_ERROR_LOG(@"%s (unexpected error - pushing package failed)", __PRETTY_FUNCTION__);
        }
    }
    
}

- (void)pushPackage:(id)package withUuid:(NSString *)uuid inSession:(MCSession *)session withResponseBlock:(id)resposeBlock {
    [self pushPackage:package withUuid:uuid inSessions:[NSArray arrayWithObject:session] withResponseBlock:resposeBlock];
}

- (void)pushPackage:(id)package inSession:(MCSession *)session withResponseBlock:(id)resposeBlock {
    [self pushPackage:package withUuid:nil inSessions:[NSArray arrayWithObject:session] withResponseBlock:resposeBlock];
}

- (void)pushPackage:(id)package inSession:(MCSession *)session  {
    [self pushPackage:package inSession:session withResponseBlock:nil];
}

- (void)pushPackage:(id)package inSessions:(NSArray *)sessions {
    [self pushPackage:package withUuid:nil inSessions:sessions withResponseBlock:nil];
}

- (void)pushPackage:(id)package withUuid:(NSString *)uuid inSession:(MCSession *)session {
    [self pushPackage:package withUuid:uuid inSession:session withResponseBlock:nil];
}

- (id)init {
    self = [super init];
    if (self) {
        self.responseBlockDictionary = [[NSMutableDictionary alloc] init];
    }
    return self;
}

@end
