//
//  PRTDrawingView.m
//  Partiture
//
//  Created by Kaarel Kruus on 11/02/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "PRTDrawingView.h"
#import "PRTRemarkView.h"
#import "PRTTextField.h"

@interface PRTDrawingView ()

@property (nonatomic) BOOL hasStartingPoint;
@property (nonatomic) CGPoint startingPoint;

@property (nonatomic) BOOL isMoving;

@property (nonatomic, strong) UIView *movingView;

@end

@implementation PRTDrawingView

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    //[super touchesBegan:touches withEvent:event];
    
    UITouch *touch = [[event allTouches] anyObject];
    self.startingPoint = [touch locationInView:self];
    self.hasStartingPoint = YES;
    
    CGPoint touchPoint = [touch locationInView:self];
    self.movingView = [self viewForPoint:touchPoint];
    if(self.movingView) {
        self.isMoving = YES;
    }
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    //[super touchesMoved:touches withEvent:event];
    
    
    UITouch *touch = [[event allTouches] anyObject];
    //NSLog(@"Touches moved!");
    
    if(self.isMoving) {
        CGPoint endingPoint = [touch locationInView:self];
        
        CGRect frame = self.movingView.frame;
        frame.origin = endingPoint;
        self.movingView.frame = frame;
        
        /*
        NSString *uuid = @"";
        if([self.movingView respondsToSelector:@selector(uuid)]) {
            uuid = [self.movingView performSelector:@selector(uuid)];
        }
         */
        
        // send event!
        //if(self.moveBlock) {
            //self.moveBlock(endingPoint, uuid);
        //}
        
    }
}


- (UIView *)viewForPoint:(CGPoint)point {
    for(UIView *subview in self.subviews) {
        if([subview isKindOfClass:[PRTRemarkView class]] ||
           [subview isKindOfClass:[PRTTextField class]]) {
            if(CGRectContainsPoint(subview.frame, point)) {
                return subview;
            }
        }
    }
    return nil;
}


- (UIView *)viewForUuid:(NSString *)uuid {
    for(UIView *subview in self.subviews) {
        if([subview isKindOfClass:[PRTRemarkView class]] ||
           [subview isKindOfClass:[PRTTextField class]]) {
            NSString *_uuid = [subview performSelector:@selector(uuid)];
            if([uuid isEqualToString:_uuid]) {
                return subview;
            }
        }
    }
    return nil;
}

- (void)didLongPressOnRemarkView:(PRTRemarkView *)remarkView {
    if(self.delegate && [self.delegate respondsToSelector:@selector(drawingView:didLongPressOnRemarkView:withUuid:)]) {
        [self.delegate drawingView:self didLongPressOnRemarkView:remarkView withUuid:remarkView.uuid];
    }
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    
    UITouch *touch = [[event allTouches] anyObject];
    CGPoint endingPoint = [touch locationInView:self];
    
    if(self.hasStartingPoint && !self.isMoving) {
        
        self.hasStartingPoint = NO;
        
        if(fabs(endingPoint.x - self.startingPoint.x) >= 20.0F) { // HIGHLIHGT
            
            CGRect rect = CGRectMake(self.startingPoint.x,
                                     self.startingPoint.y,
                                     (endingPoint.x - self.startingPoint.x),
                                     40.0F); // TODO: configure
            
            NSString *uuid = [PRTUtils generateUUID];
            
            PRTRemarkView *remarkView = [[PRTRemarkView alloc] initWithFrame:rect];
            remarkView.uuid = uuid;
            remarkView.delegate = self;
            remarkView.backgroundColor = [UIColor greenColor]; // TODO: configurable
            remarkView.alpha = 0.6;
            remarkView.userInteractionEnabled = YES;
            
            BOOL shouldDrawRemarkView = YES;
            
            if(self.delegate && [self.delegate respondsToSelector:@selector(drawingView:shouldDrawRemarkView:withUuid:)]) {
                shouldDrawRemarkView = [self.delegate drawingView:self shouldDrawRemarkView:remarkView withUuid:uuid];
            }
            
            if(shouldDrawRemarkView) {
                [self addSubview:remarkView];
                if(self.delegate && [self.delegate respondsToSelector:@selector(drawingView:didDrawRemarkView:withUuid:)]) {
                    [self.delegate drawingView:self didDrawRemarkView:remarkView withUuid:uuid];
                }
            }
            
            
            
        } else { // TEXT
            
            /*
            [UIView animateWithDuration:0.5F animations:^{
                [self.superview setTransform:CGAffineTransformMakeScale(1.0, 1.0)];
            }];
             */
            /*
            CGRect rect = CGRectMake(self.startingPoint.x,
                                     self.startingPoint.y,
                                     300.0F,
                                     40.0F);
            
            PRTTextField *txtField = [[PRTTextField alloc] initWithFrame:rect];
            txtField.autocorrectionType = UITextAutocorrectionTypeNo;
            txtField.backgroundColor = [UIColor clearColor];
            #if TARGET_MASTER
            txtField.textColor = [UIColor redColor];
            #else
            txtField.textColor = [UIColor blueColor];
            #endif
            txtField.borderStyle = UITextBorderStyleNone;
            //txtField.text = @"IMPORTANT!";
            txtField.enabled = YES;
            txtField.delegate = self;
            txtField.uuid = [self generateUUIDString];
            [self addSubview:txtField];
            
            [txtField becomeFirstResponder];
             */
        }
        
    }
    
    self.isMoving = NO;
    self.movingView = nil;
}

/*
- (void)textFieldDidEndEditing:(UITextField *)textField {
    textField.enabled = NO;
    if(self.drawTextFieldBlock) {
        self.drawTextFieldBlock(textField, ((PRTTextField *)textField).uuid);
    }
}
*/

@end
