//
//  PRTDrawingView.m
//  Partiture
//
//  Created by Kaarel Kruus on 11/02/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "PRTDrawingView.h"
#import "PRTView.h"
#import "PRTTextField.h"

@interface PRTDrawingView ()

@property (nonatomic) BOOL hasStartingPoint;
@property (nonatomic) CGPoint startingPoint;

@property (nonatomic) BOOL isMoving;

@property (nonatomic, strong) UIView *movingView;

@end

@implementation PRTDrawingView

- (NSString *)generateUUIDString
{
    CFUUIDRef theUUID = CFUUIDCreate(NULL);
    CFStringRef string = CFUUIDCreateString(NULL, theUUID);
    CFRelease(theUUID);
    return (__bridge NSString *)string;
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    //[super touchesBegan:touches withEvent:event];
    
    UITouch *touch = [[event allTouches] anyObject];
    self.startingPoint = [touch locationInView:self];
    self.hasStartingPoint = YES;
    
    CGPoint touchPoint = [touch locationInView:self];
    
    self.movingView = [self viewForPoint:touchPoint];
    
    if(self.movingView) {
        self.isMoving = YES;
    }
    
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    //[super touchesMoved:touches withEvent:event];
    
    UITouch *touch = [[event allTouches] anyObject];
    //NSLog(@"Touches moved!");
    
    if(self.isMoving) {
        CGPoint endingPoint = [touch locationInView:self];
        
        CGRect frame = self.movingView.frame;
        frame.origin = endingPoint;
        self.movingView.frame = frame;
        
        NSString *guid = @"";
        if([self.movingView respondsToSelector:@selector(guid)]) {
            guid = [self.movingView performSelector:@selector(guid)];
            //NSLog(@"GUID: %@", guid);
        }
        
        // send event!
        if(self.moveBlock) {
            
            self.moveBlock(endingPoint, guid);
        }
        
    }
}

- (UIView *)viewForPoint:(CGPoint)point {
    for(UIView *subview in self.subviews) {
        if([subview isKindOfClass:[PRTView class]] ||
           [subview isKindOfClass:[PRTTextField class]]) {
            if(CGRectContainsPoint(subview.frame, point)) {
                return subview;
            }
        }
    }
    return nil;
}

- (UIView *)viewForGuid:(NSString *)_guid {
    for(UIView *subview in self.subviews) {
        if([subview isKindOfClass:[PRTView class]] ||
           [subview isKindOfClass:[PRTTextField class]]) {
            
            NSString *guid = [subview performSelector:@selector(guid)];
            if([_guid isEqualToString:guid]) {
                return subview;
            }
        }
    }
    return nil;
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    
    UITouch *touch = [[event allTouches] anyObject];
    CGPoint endingPoint = [touch locationInView:self];
    
    if(self.hasStartingPoint && !self.isMoving) {
        
        self.hasStartingPoint = NO;
        
        if((endingPoint.x - self.startingPoint.x) >= 20.0F) { // HIGHLIHGT
            
            CGRect rect = CGRectMake(self.startingPoint.x,
                                     self.startingPoint.y,
                                     (endingPoint.x - self.startingPoint.x),
                                     40.0F);
            
            PRTView *view = [[PRTView alloc] initWithFrame:rect];
            #if TARGET_MASTER
            view.backgroundColor = [UIColor redColor];
            #else
            view.backgroundColor = [UIColor blueColor];
            #endif
            view.alpha = 0.6;
            view.userInteractionEnabled = YES;
            view.guid = [self generateUUIDString];
            [self addSubview:view];
            
            if(self.drawViewBlock) {
                self.drawViewBlock(view, view.guid);
            }
            
        } else { // TEXT
            
            /*
            [UIView animateWithDuration:0.5F animations:^{
                [self.superview setTransform:CGAffineTransformMakeScale(1.0, 1.0)];
            }];
             */
            
            CGRect rect = CGRectMake(self.startingPoint.x,
                                     self.startingPoint.y,
                                     300.0F,
                                     40.0F);
            
            PRTTextField *txtField = [[PRTTextField alloc] initWithFrame:rect];
            txtField.autocorrectionType = UITextAutocorrectionTypeNo;
            txtField.backgroundColor = [UIColor clearColor];
            #if TARGET_MASTER
            txtField.textColor = [UIColor redColor];
            #else
            txtField.textColor = [UIColor blueColor];
            #endif
            txtField.borderStyle = UITextBorderStyleNone;
            //txtField.text = @"IMPORTANT!";
            txtField.enabled = YES;
            txtField.delegate = self;
            txtField.guid = [self generateUUIDString];
            [self addSubview:txtField];
            
            [txtField becomeFirstResponder];
        }
        
    }
    
    self.isMoving = NO;
    self.movingView = nil;
}

- (void)textFieldDidEndEditing:(UITextField *)textField {
    textField.enabled = NO;
    if(self.drawTextFieldBlock) {
        self.drawTextFieldBlock(textField, ((PRTTextField *)textField).guid);
    }
}

@end
