//
//  PRTOrganizerViewController.m
//  Partiture
//
//  Created by Kaarel Kruus on 08/02/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "PRTOrganizerViewController.h"
#import "TBXML+HTTP.h"
#import "PRTFileManager.h"
#import <MBProgressHUD/MBProgressHUD.h>
#import <AFNetworking/AFNetworking.h>
#import "SSZipArchive.h"
#import "PRTAppDelegate.h"
#import "ReaderDocument.h"
#import "Score+Additions.h"
#import "Layer.h"

#import "PRTWorkspaceViewController.h"

@interface PRTOrganizerViewController ()

@property (nonatomic, strong) NSMutableArray *downloadedFileSets; // all filesets downloaded to the device
@property (nonatomic, strong) NSMutableArray *availableFileSets; // all filesets available via the service
@property (nonatomic, strong) NSMutableArray *freshFileSets; // all filesets available, but not downlaoded

@end

@implementation PRTOrganizerViewController

#define FRESH_FILESETS_SECTION 0
#define DOWNLOADED_FILESETS_SECTION 1

- (id)init {
    self = [super initWithNibName:@"PRTOrganizerViewController" bundle:nil];
    if (self) {
        self.tabBarItem.title = @"Organizer";
        self.tabBarItem.image = [UIImage imageNamed:@"icon-organizer.png"];
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self refreshFileSets];
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    return (indexPath.section == DOWNLOADED_FILESETS_SECTION);
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if(indexPath.section == DOWNLOADED_FILESETS_SECTION) {
        if (editingStyle == UITableViewCellEditingStyleDelete) {
            
            NSString *checksum = [[self.downloadedFileSets objectAtIndex:indexPath.row] checksum];
            
            NSString *path = [[PRTFileManager sharedManager] fileSetsPath];
            
            [[NSFileManager defaultManager] removeItemAtPath:[NSString stringWithFormat:@"%@/%@", path, checksum]
                                                       error:nil];
            
            /* delete from database */
            [self deleteFileSet:[self.downloadedFileSets objectAtIndex:indexPath.row]];
            
            [self refreshFileSets];
        }
    }
}

- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText {
    [self.tableView reloadData];
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar {
    [self.tableView reloadData];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 44.0F;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    return (section == DOWNLOADED_FILESETS_SECTION) ? @"Downloaded scores" : @"Downloadable scores";
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return (section == DOWNLOADED_FILESETS_SECTION) ? self.downloadedFileSets.count : self.freshFileSets.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    NSString *cellIdentifier = @"fileSetListCell";
    
    UITableViewCell *cell = (UITableViewCell *)[tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    
    if(!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle
                                      reuseIdentifier:cellIdentifier];
    }
    
    if(indexPath.section == DOWNLOADED_FILESETS_SECTION) {
        
        Score *score = [self.downloadedFileSets objectAtIndex:indexPath.row];
        cell.textLabel.text = score.title;
        cell.detailTextLabel.text = [NSString stringWithFormat:@"Composer: %@ / Lyrics: %@",
                                     [score composerString], [score lyricistString]];
        
        if([self string:score.title containsString:self.searchBar.text] ||
           [self string:[score composerString] containsString:self.searchBar.text]) {
            cell.backgroundColor = [UIColor yellowColor];
        } else {
            cell.backgroundColor = [UIColor clearColor];
        }
        
    } else {
        
        Score *aScore = [self.freshFileSets objectAtIndex:indexPath.row];
        cell.textLabel.text = aScore.title;
        cell.detailTextLabel.text = [NSString stringWithFormat:@"Composer: %@ / Lyrics: %@",
                                     [aScore composerString], [aScore lyricistString]];
        
        if([self string:aScore.title containsString:self.searchBar.text] ||
           [self string:[aScore composerString] containsString:self.searchBar.text]) {
            cell.backgroundColor = [UIColor yellowColor];
        } else {
            cell.backgroundColor = [UIColor clearColor];
        }
    }
    
    return cell;
}

- (BOOL)string:(NSString *)str1 containsString:(NSString *)str2 {
    return (([[str1 lowercaseString] rangeOfString:[str2 lowercaseString]].location != NSNotFound) &&
            str1 &&
            str2);
}

- (void)dismissReaderViewController:(ReaderViewController *)viewController {
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if(indexPath.section == DOWNLOADED_FILESETS_SECTION) {
        
        Score *score = [self.downloadedFileSets objectAtIndex:indexPath.row];
        NSString *filename = [[PRTFileManager sharedManager] filePathForChecksum:score.checksum];
        NSString *filepath = [NSString stringWithFormat:@"%@/%@/%@",
                              [[PRTFileManager sharedManager] fileSetsPath], score.checksum, filename];
        
        ReaderDocument *document = [ReaderDocument withDocumentFilePath:filepath password:nil];
        
        if(document) {
            
            #if TARGET_MASTER
            PRTAdvertiserController *rvc = [[PRTAdvertiserController alloc] initWithReaderDocument:document];
            #else
            PRTBrowserController *rvc = [[PRTBrowserController alloc] initWithReaderDocument:document];
            #endif
            
            rvc.delegate = self;
            rvc.score = score;
            [self presentViewController:rvc animated:YES completion:nil];
        }
        
        /*
        PRTWorkspaceViewController *workspaceViewController = [[PRTWorkspaceViewController alloc] init];
        [self presentViewController:workspaceViewController animated:YES completion:nil];
        */
        
    } else {
        
        /* trigger download */
        
        MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
        
        PRTFileManager *fileManager = [PRTFileManager sharedManager];
        
        NSURLSessionConfiguration *configuration = [NSURLSessionConfiguration defaultSessionConfiguration];
        AFURLSessionManager *manager = [[AFURLSessionManager alloc] initWithSessionConfiguration:configuration];
        
        Score *aScore = [self.freshFileSets objectAtIndex:indexPath.row];
        
        NSURL *URL = [NSURL URLWithString:[aScore href]];
        NSURLRequest *request = [NSURLRequest requestWithURL:URL];
        
        NSString *fileSetsDirPath = [fileManager fileSetsPath];
        NSString *zipFileName = [NSString stringWithFormat:@"%@.zip", [aScore checksum]];
        NSString *zipFilePath = [fileSetsDirPath stringByAppendingFormat:@"/%@", zipFileName];
        NSString *unzipDirPath = [fileSetsDirPath stringByAppendingFormat:@"/%@", [aScore checksum]];
        
        NSURLSessionDownloadTask *downloadTask = [manager downloadTaskWithRequest:request
                                                                         progress:nil
                                                                      destination:^NSURL *(NSURL *targetPath, NSURLResponse *response) {
                                                                          return [NSURL fileURLWithPath:zipFilePath];
                                                                          
                                                                      } completionHandler:^(NSURLResponse *response, NSURL *filePath, NSError *error) {
                                                                          
                                                                          /* unzip */
                                                                          NSString *input = zipFilePath;
                                                                          NSString *output = unzipDirPath;
                                                                          
                                                                          NSError *err;
                                                                          [[NSFileManager defaultManager] createDirectoryAtPath:output
                                                                                                    withIntermediateDirectories:NO
                                                                                                                     attributes:nil
                                                                                                                          error:&err];
                                                                          
                                                                          [SSZipArchive unzipFileAtPath:input toDestination:output];
                                                                          
                                                                          /* delete zip file */
                                                                          [[[NSFileManager alloc] init] removeItemAtPath:input error:nil];
                                                                          
                                                                          /* insert into database */
                                                                          Score *score = [NSEntityDescription insertNewObjectForEntityForName:@"Score" inManagedObjectContext:[APPDELEGATE managedObjectContext]];
                                                                          score.title = aScore.title;
                                                                          score.checksum = aScore.checksum;
                                                                          score.origin = aScore.origin;
                                                                          score.href = aScore.href;
                                                                          //score.composers // TODO
                                                                          //score.lyricists // TODO
                                                                          //score.arrangers // TODO
                                                                          [APPDELEGATE saveContext];
                                                                          
                                                                          /* create public layer */
                                                                          Layer *layer = [NSEntityDescription insertNewObjectForEntityForName:@"Layer" inManagedObjectContext:[APPDELEGATE managedObjectContext]];
                                                                          layer.name = @"Public";
                                                                          layer.isPublic = @(YES);
                                                                          layer.guid = [PRTUtils generateGUID];
                                                                          layer.layerOf = score;
                                                                          [APPDELEGATE saveContext];
                                                                          
                                                                          dispatch_async(dispatch_get_main_queue(), ^{
                                                                              [hud hide:YES];
                                                                              [self refreshFileSets];
                                                                          });
                                                                      }];
        
        [manager setDownloadTaskDidWriteDataBlock:^(NSURLSession *session, NSURLSessionDownloadTask *downloadTask, int64_t bytesWritten, int64_t totalBytesWritten, int64_t totalBytesExpectedToWrite) {
            double d_percentage = ((double)totalBytesWritten / (double)totalBytesExpectedToWrite);
            int i_percentage = (int)(d_percentage * 100);
            hud.labelText = [NSString stringWithFormat:@"Downloading...%i%%", i_percentage];
        }];
        [downloadTask resume];
    }
}


- (void)loadAvailableFileSets:(PRTBlock)completionBlock {
    
    // clear current array
    self.availableFileSets = [[NSMutableArray alloc] init];
    
    NSString *serviceUrl = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"ServiceUrl"];
    
    TBXML *tbxml = [[TBXML alloc] initWithURL:[NSURL URLWithString:serviceUrl] success:^(TBXML *tbxml) {
        
        TBXMLElement *root = tbxml.rootXMLElement;
        
        TBXMLElement *entry = [TBXML childElementNamed:@"fileset" parentElement:root];
        
        while (entry != nil) {
            
            Score *aScore = [Score unassociatedObject];
            aScore.title = [TBXML valueOfAttributeNamed:@"title" forElement:entry];
            aScore.checksum = [TBXML valueOfAttributeNamed:@"checksum" forElement:entry];
            aScore.href = [TBXML valueOfAttributeNamed:@"href" forElement:entry];
            aScore.origin = [TBXML valueOfAttributeNamed:@"origin" forElement:entry];
            //aScore.composers // TODO
            //aScore.lyricists // TODO
            //aScore.arrangers // TODO
            [self.availableFileSets addObject:aScore];
            
            entry = [TBXML nextSiblingNamed:@"fileset" searchFromElement:entry];
        }
        
        if(completionBlock) {
            completionBlock();
        }
    } failure:^(TBXML *tbxml, NSError *error) {
        if(completionBlock) {
            completionBlock();
        }
    }];
}

- (void)loadDownloadedFileSets {
    
    self.downloadedFileSets = [[NSMutableArray alloc] init];
    
    NSManagedObjectContext *context = [APPDELEGATE managedObjectContext];
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Score" inManagedObjectContext:context];
    [fetchRequest setEntity:entity];
    
    NSError *error = nil;
    NSArray *fetchedObjects = [context executeFetchRequest:fetchRequest error:&error];
    
    for (Score *score in fetchedObjects) {
        [self.downloadedFileSets addObject:score];
    }
}

- (void)deleteFileSet:(Score *)score {
    //NSLog(@"Delete fileset: %@", fileset.title);
    [[APPDELEGATE managedObjectContext] deleteObject:score];
}

- (void)filterFreshFileSets {
    
    self.freshFileSets = [[NSMutableArray alloc] init];
    
    // all the filesets not being downloaded and being available are considered as "fresh"
    for (Score *availableFileSet in self.availableFileSets) {
        if(![self isFileSetDownloaded:availableFileSet]) {
            [self.freshFileSets addObject:availableFileSet];
        }
    }
}

- (BOOL)isFileSetDownloaded:(Score *)score {
    for (Score *downloadedFileSet in self.downloadedFileSets) {
        if([score.checksum isEqualToString:downloadedFileSet.checksum]) {
            return YES;
        }
    }
    return NO;
}

- (void)refreshFileSets {
    [self loadAvailableFileSets:^{
        [self loadDownloadedFileSets];
        [self filterFreshFileSets];
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.tableView reloadData];
        });
    }];
}

@end
