//
//  PRTOrganizerViewController.m
//  Partiture
//
//  Created by Kaarel Kruus on 08/02/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "PRTOrganizerViewController.h"
#import "TBXML+HTTP.h"
#import "PRTFileManager.h"
#import <MBProgressHUD/MBProgressHUD.h>
#import <AFNetworking/AFNetworking.h>
#import "SSZipArchive.h"
#import "PRTAppDelegate.h"
#import "ReaderDocument.h"
#import "Score+Additions.h"
#import "Layer.h"

#import "PRTWorkspaceViewController.h"

#import "PRTRoomViewController.h"

#import "DAOManager.h"

#import "Bundle+Additions.h"
#import "Member+Additions.h"
#import "Participation+Additions.h"
#import "Membership+Enums.h"
#import "Membership+Additions.h"

#import "PRTWorkspaceViewController.h"

@interface PRTOrganizerViewController ()

@property (nonatomic, strong) NSArray *downloadedFileSets; // all filesets downloaded to the device
@property (nonatomic, strong) NSMutableArray *availableFileSets; // all filesets available via the service
@property (nonatomic, strong) NSMutableArray *freshFileSets; // all filesets available, but not downlaoded

@end

@implementation PRTOrganizerViewController

#define FRESH_FILESETS_SECTION 0
#define DOWNLOADED_FILESETS_SECTION 1

#define SEGUE_pushRoomViewController @"pushRoomViewController"

/*
- (id)init {
    self = [super initWithNibName:@"PRTOrganizerViewController" bundle:nil];
    if (self) {
        self.tabBarItem.title = @"Organizer";
        self.tabBarItem.image = [UIImage imageNamed:@"icon-organizer.png"];
    }
    return self;
}
*/

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // add observers
    NSNotificationCenter *noticicationCenter = [NSNotificationCenter defaultCenter];
    [noticicationCenter addObserver:self
                           selector:@selector(onApplicationDidBecomeActive:)
                               name:PRTApplicationDidBecomeActiveNotification
                             object:nil];
    
    [self refreshFileSets];
}

- (void)onApplicationDidBecomeActive:(NSNotification *)notification {
    
    // check if we should restore some room/session
    User *appUser = [APPDELEGATE appUser];
    if(appUser) {
        
        // we know the member, but not yet the room/session to be restored
        Member *member = [Member fetchMemberWithToken:appUser.token];
        if(member) {
            
            // TODO: order by date
            NSArray *activeParticipations = [member fetchActiveParticipations];
            if(activeParticipations.count > 0) {
                
                Participation *activeParticipation = activeParticipations[0];
                Room *room = activeParticipation.session.room;
                
                // check the role
                Membership *memberShip = [Membership fetchMembershipForMember:member inRoom:room];
                
                if([memberShip.role isEqual:@(MembershipRoleRoomOwner)]) {
                    
                    // check, maybe the workspace is already created?
                    UIViewController *visibleViewController = self.navigationController.viewControllers.lastObject;
                    if([visibleViewController isKindOfClass:[PRTRoomViewController class]]) {
                        
                        if([visibleViewController.presentedViewController isKindOfClass:[PRTWorkspaceViewController class]]) {
                            
                            PRTWorkspaceViewController *wsvc = (PRTWorkspaceViewController *)visibleViewController.presentedViewController;
                            if([wsvc.currentSession isEqual:activeParticipation.session]) {
                                NSLog(@"ALREADY RESTORED, END HERE!");
                            } else {
                                // TODO: ask user if the workspace should we switched
                            }
                            
                        } else {
                            PRTRoomViewController *rvc = (PRTRoomViewController *)visibleViewController;
                            [rvc openWorkspaceInRoom:room withSession:activeParticipation.session asOwner:YES];
                        }
                        
                    } else {
                        
                        // push (no animation!) RoomViewController
                        Bundle *bundle = room.bundle;
                        [self performSegueWithIdentifier:SEGUE_pushRoomViewController sender:bundle];
                        
                        // present WorkspaceViewController
                        PRTRoomViewController *rvc = self.navigationController.viewControllers.lastObject;
                        [rvc openWorkspaceInRoom:room withSession:activeParticipation.session asOwner:YES];
                        
                    }

                } else {
                    NSLog(@"Restore as guest!");
                }
                
            } else {
                // TODO: handle
                NSLog(@"INFO: No active participation!");
            }
            
        } else {
            // TODO: handle
            NSLog(@"INFO: No member!");
        }
    } else {
        // TODO: handle
    }
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    return (indexPath.section == DOWNLOADED_FILESETS_SECTION);
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if(indexPath.section == DOWNLOADED_FILESETS_SECTION) {
        if (editingStyle == UITableViewCellEditingStyleDelete) {
            
            NSString *checksum = [[self.downloadedFileSets objectAtIndex:indexPath.row] token];
            
            NSString *path = [[PRTFileManager sharedManager] fileSetsPath];
            
            [[NSFileManager defaultManager] removeItemAtPath:[NSString stringWithFormat:@"%@/%@", path, checksum]
                                                       error:nil];
            
            /* delete from database */
            [self deleteFileSet:[self.downloadedFileSets objectAtIndex:indexPath.row]];
            
            [self refreshFileSets];
        }
    }
}

- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText {
    [self.tableView reloadData];
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar {
    [self.tableView reloadData];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 44.0F;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    return (section == DOWNLOADED_FILESETS_SECTION) ? @"Downloaded scores" : @"Downloadable scores";
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return (section == DOWNLOADED_FILESETS_SECTION) ? self.downloadedFileSets.count : self.freshFileSets.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    NSString *cellIdentifier = @"fileSetListCell";
    
    UITableViewCell *cell = (UITableViewCell *)[tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    
    if(!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle
                                      reuseIdentifier:cellIdentifier];
    }
    
    if(indexPath.section == DOWNLOADED_FILESETS_SECTION) {
        
        Score *score = [self.downloadedFileSets objectAtIndex:indexPath.row];
        cell.textLabel.text = score.title;
        cell.detailTextLabel.text = [NSString stringWithFormat:@"Composer: %@ / Lyrics: %@", [score composerString], [score lyricistString]];
        
        if([self string:score.title containsString:self.searchBar.text] ||
           [self string:[score composerString] containsString:self.searchBar.text]) {
            cell.backgroundColor = [UIColor yellowColor];
        } else {
            cell.backgroundColor = [UIColor clearColor];
        }
        
    } else {
        
        Score *aScore = [self.freshFileSets objectAtIndex:indexPath.row];
        cell.textLabel.text = aScore.title;
        cell.detailTextLabel.text = [NSString stringWithFormat:@"Composer: %@ / Lyrics: %@", [aScore composerString], [aScore lyricistString]];
        
        if([self string:aScore.title containsString:self.searchBar.text] ||
           [self string:[aScore composerString] containsString:self.searchBar.text]) {
            cell.backgroundColor = [UIColor yellowColor];
        } else {
            cell.backgroundColor = [UIColor clearColor];
        }
    }
    
    return cell;
}

- (BOOL)string:(NSString *)str1 containsString:(NSString *)str2 {
    return (([[str1 lowercaseString] rangeOfString:[str2 lowercaseString]].location != NSNotFound) &&
            str1 &&
            str2);
}

- (void)dismissReaderViewController:(ReaderViewController *)viewController {
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([segue.identifier isEqualToString:SEGUE_pushRoomViewController]) {
        PRTRoomViewController *roomViewController = segue.destinationViewController;
        roomViewController.bundle = (Bundle *)sender;
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if(indexPath.section == DOWNLOADED_FILESETS_SECTION) {
        
        Score *score = [self.downloadedFileSets objectAtIndex:indexPath.row];
        Bundle *bundle = [[APPDELEGATE appUser] bundleForScore:score];
        
        if(bundle) {
            [self performSegueWithIdentifier:SEGUE_pushRoomViewController sender:bundle];
        } else {
            // TODO: error management
        }
        
    } else {
        
        /* trigger download */
        
        MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
        
        PRTFileManager *fileManager = [PRTFileManager sharedManager];
        
        NSURLSessionConfiguration *configuration = [NSURLSessionConfiguration defaultSessionConfiguration];
        AFURLSessionManager *manager = [[AFURLSessionManager alloc] initWithSessionConfiguration:configuration];
        
        Score *aScore = [self.freshFileSets objectAtIndex:indexPath.row];
        
        //NSURL *URL = [NSURL URLWithString:[aScore href]];
        NSURL *URL = [NSURL URLWithString:[NSString stringWithFormat:SERVICE_CONFIGURATION[@"FetchUrl"], aScore.token]];
        NSURLRequest *request = [NSURLRequest requestWithURL:URL];
        
        NSString *fileSetsDirPath = [fileManager fileSetsPath];
        NSString *zipFileName = [NSString stringWithFormat:@"%@.zip", [aScore token]];
        NSString *zipFilePath = [fileSetsDirPath stringByAppendingFormat:@"/%@", zipFileName];
        NSString *unzipDirPath = [fileSetsDirPath stringByAppendingFormat:@"/%@", [aScore token]];
        
        NSURLSessionDownloadTask *downloadTask = [manager downloadTaskWithRequest:request
                                                                         progress:nil
                                                                      destination:^NSURL *(NSURL *targetPath, NSURLResponse *response) {
                                                                          return [NSURL fileURLWithPath:zipFilePath];
                                                                          
                                                                      } completionHandler:^(NSURLResponse *response, NSURL *filePath, NSError *error) {
                                                                          
                                                                          /* unzip */
                                                                          NSString *input = zipFilePath;
                                                                          NSString *output = unzipDirPath;
                                                                          
                                                                          NSError *err;
                                                                          [[NSFileManager defaultManager] createDirectoryAtPath:output
                                                                                                    withIntermediateDirectories:NO
                                                                                                                     attributes:nil
                                                                                                                          error:&err];
                                                                          
                                                                          BOOL success = [SSZipArchive unzipFileAtPath:input toDestination:output];
                                                                          
                                                                          /* delete zip file */
                                                                          [[[NSFileManager alloc] init] removeItemAtPath:input error:nil];
                                                                          
                                                                          if(success) {
                                                                              Bundle *bundle;
                                                                              bundle = [Bundle bundleWithAbstractScore:aScore
                                                                                                               forUser:[APPDELEGATE appUser]];
                                                                          } else {
                                                                              PRT_ERROR_LOG(@"%s (unexpected error - unzipping fileset failed)", __PRETTY_FUNCTION__);
                                                                          }
                                                                          
                                                                          DISPATCH_ASYNC_MAIN({
                                                                              [hud hide:YES];
                                                                              [self refreshFileSets];
                                                                          });
                                                                      }];
        
        [manager setDownloadTaskDidWriteDataBlock:^(NSURLSession *session, NSURLSessionDownloadTask *downloadTask, int64_t bytesWritten, int64_t totalBytesWritten, int64_t totalBytesExpectedToWrite) {
            double d_percentage = ((double)totalBytesWritten / (double)totalBytesExpectedToWrite);
            int i_percentage = (int)(d_percentage * 100);
            hud.labelText = [NSString stringWithFormat:@"Downloading...%i%%", i_percentage];
        }];
        [downloadTask resume];
    }
}


- (void)loadAvailableFileSets:(PRTBlock)completionBlock {
    
    // clear current array
    self.availableFileSets = [[NSMutableArray alloc] init];
    
    [[DAOManager sharedManager] loadAvailableAbstractScoreListForUser:[APPDELEGATE appUser]
                                                  withCompletionBlock:^(id result, NSError *error) {
                                                      
        if(!error && [result isKindOfClass:[NSMutableArray class]]) {
            self.availableFileSets = (NSMutableArray *)result;
        } else {
            //TODO: error management
        }
        
        if(completionBlock) {
            completionBlock();
        }
    }];
}

- (void)loadDownloadedFileSets {
    // clear (refresh) current array
    self.downloadedFileSets = [[APPDELEGATE appUser] scores];
}

- (void)deleteFileSet:(Score *)score {
    //NSLog(@"Delete fileset: %@", fileset.title);
    [[APPDELEGATE managedObjectContext] deleteObject:score];
}

- (void)filterFreshFileSets {
    
    self.freshFileSets = [[NSMutableArray alloc] init];
    
    // all the filesets not being downloaded and being available are considered as "fresh"
    for (Score *availableFileSet in self.availableFileSets) {
        if(![self isFileSetDownloaded:availableFileSet]) {
            [self.freshFileSets addObject:availableFileSet];
        }
    }
}

- (BOOL)isFileSetDownloaded:(Score *)score {
    for (Score *downloadedFileSet in self.downloadedFileSets) {
        if([score.token isEqualToString:downloadedFileSet.token]) {
            return YES;
        }
    }
    return NO;
}

- (void)refreshFileSets {
    [self loadAvailableFileSets:^{
        [self loadDownloadedFileSets];
        [self filterFreshFileSets];
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.tableView reloadData];
        });
    }];
}

@end
