//
//  PRTRoomViewController.m
//  Partiture
//
//  Created by Kaarel Kruus on 15/03/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "PRTRoomViewController.h"
#import "PRTRoomDetailViewController.h"
#import "Session.h"
#import "PRTWorkspaceViewController.h"
#import "PRTCommunicationManager.h"
#import "PRTCOMListener.h"
#import <MultipeerConnectivity/MultipeerConnectivity.h>
#import "PRTAppDelegate.h"
#import "PRTRoomInfo.h"
#import "PRTAlertView.h"
#import "PRTFileManager.h"
#import "Bundle+Additions.h"

#define MY_CREATED_ROOMS_SECTION 0
#define MY_JOINED_ROOMS_SECTION 1
#define VISIBLE_ROOMS_SECTION 2

#define SEGUE_presentRoomDetailViewController @"presentRoomDetailViewController"

@interface PRTRoomViewController ()

@property (nonatomic, strong) NSArray *visibleRooms;

@end

@implementation PRTRoomViewController

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([segue.identifier isEqualToString:SEGUE_presentRoomDetailViewController]) {
        PRTRoomDetailViewController *roomDetailViewController = segue.destinationViewController;
        roomDetailViewController.bundle = self.bundle;
        roomDetailViewController.dismissBlock = ^{
            DISPATCH_ASYNC_MAIN({
                [self.tableView reloadData];
            });
        };
    }
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // add observers
    NSNotificationCenter *notificationCenter = [NSNotificationCenter defaultCenter];
    [notificationCenter addObserver:self
                           selector:@selector(roomSessionsUpdate:)
                               name:PRTCOMListenerRoomSessionsUpdateNotification
                             object:nil];
    
    self.visibleRooms = [COM_LISTENER visibleRooms];
    [self.tableView reloadData];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [APPDELEGATE startListening];
}

- (void)roomSessionsUpdate:(NSNotification *)notification {
    DISPATCH_ASYNC_MAIN({
        self.visibleRooms = [COM_LISTENER visibleRooms];
        [self.tableView reloadData];
    });
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 3;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    
    NSArray *ownedRooms; NSArray *guestRooms;
    [self.bundle loadOwnedRooms:&ownedRooms andGuestRooms:&guestRooms];
    
    if(section == MY_CREATED_ROOMS_SECTION) {
        return ownedRooms.count;
    } else if(section == MY_JOINED_ROOMS_SECTION) {
        return guestRooms.count;
    } else if(section == VISIBLE_ROOMS_SECTION) {
        return self.visibleRooms.count;
    }
    
    return 0;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    if(section == MY_CREATED_ROOMS_SECTION) {
        return @"My created rooms";
    } else if(section == MY_JOINED_ROOMS_SECTION) {
        return @"My joined rooms";
    } else if(section == VISIBLE_ROOMS_SECTION) {
        return @"Visible rooms";
    } else {
        return @"";
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *cellID = @"roomCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellID];
    
    if(!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellID];
    }
    
    NSArray *ownedRooms; NSArray *guestRooms;
    [self.bundle loadOwnedRooms:&ownedRooms andGuestRooms:&guestRooms];
    
    if(indexPath.section == MY_CREATED_ROOMS_SECTION) {
        Room *room = ownedRooms[indexPath.row];
        cell.textLabel.text = room.name;
    }
    else if(indexPath.section == MY_JOINED_ROOMS_SECTION) {
        Room *room = guestRooms[indexPath.row];
        cell.textLabel.text = room.name;
    }
    else if(indexPath.section == VISIBLE_ROOMS_SECTION) {
        PRTRoomInfo *roomInfo = self.visibleRooms[indexPath.row];
        cell.textLabel.text = roomInfo.roomName;
        cell.detailTextLabel.text = roomInfo.scoreToken;
    }
    else {
        // TODO: what here?
    }
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    NSArray *ownedRooms; NSArray *guestRooms;
    [self.bundle loadOwnedRooms:&ownedRooms andGuestRooms:&guestRooms];
    
    if(indexPath.section == MY_CREATED_ROOMS_SECTION) {
        
        Room *room = ownedRooms[indexPath.row];
        [self openWorkspaceInRoom:room asOwner:YES];
        
    }
    else if(indexPath.section == MY_JOINED_ROOMS_SECTION) {
        
        Room *room = guestRooms[indexPath.row];
        [self openWorkspaceInRoom:room asOwner:NO];
        
    }
    else if(indexPath.section == VISIBLE_ROOMS_SECTION) {
        
        PRTRoomInfo *roomInfo = self.visibleRooms[indexPath.row];
        Score *score = [Score fetchScoreWithToken:roomInfo.scoreToken];
        
        if(!score) {
            
            PRTAlertView *alert = [[PRTAlertView alloc] initWithTitle:@"Sorry!"
                                                              message:@"To enter the room, you need to download the score first!"
                                                             delegate:nil
                                                    cancelButtonTitle:@"OK"
                                                    otherButtonTitles: nil];
            [alert show];
            
        } else {
            
            // we have the score, so the join request can be sent
            User *appUser = [APPDELEGATE appUser];
            
            // we also need the passkey in order to make the join request
            PRTAlertView *alert = [[PRTAlertView alloc] init];
            __block PRTAlertView *_alert = alert;
            
            alert.alertViewStyle = UIAlertViewStyleSecureTextInput;
            alert.title = @"Please insert the passkey";
            alert.message = @"In order to join the room, you have to insert the passkey provided by the room owner";
            
            [alert addButtonWithTitle:@"Cancel"];
            [alert addButtonWithTitle:@"Join"];
            
            
            [alert setTappedButtonBlock:^(NSInteger index) {
                if(index == 1) {
                    
                    NSString *passkey = [_alert textFieldAtIndex:0].text;
                    
                    [COM_LISTENER pushRoomJoinRequest:[PRTRoomJoinRequestInfo infoWithUserToken:appUser.token
                                                                                        andName:appUser.name
                                                                                     andPasskey:passkey
                                                                                  responseBlock:^(BOOL joined) {
                                                                                      DISPATCH_ASYNC_MAIN({
                                                                                          if(joined) {
                                                                                              
                                                                                              // create the room locally
                                                                                              Bundle *bundle = [appUser bundleForScore:score];
                                                                                              if(bundle) {
                                                                                                  Room *room = [Room roomWithName:roomInfo.roomName
                                                                                                                             uuid:roomInfo.roomUuid
                                                                                                                       andPasskey:passkey
                                                                                                                        forBundle:bundle];
                                                                                                  [self openWorkspaceInRoom:room asOwner:NO];
                                                                                              } else {
                                                                                                  PRT_ERROR_LOG(@"%s (unexpected error - can not retrieve a bundle for the score)", __PRETTY_FUNCTION__);
                                                                                              }
                                                                                              
                                                                                          } else {
                                                                                              // TODO: notify the user
                                                                                          }
                                                                                      });
                                                                                  }] inRoom:roomInfo];
                }
            }];
            
            [alert show];
        }
    }
}

- (void)openWorkspaceInRoom:(Room *)room withSession:(Session *)session asOwner:(BOOL)owner {
    
    Score *score = room.bundle.score;
    
    NSString *filename = [[PRTFileManager sharedManager] filePathForChecksum:score.token];
    NSString *filepath = [NSString stringWithFormat:@"%@/%@/%@",
                          [[PRTFileManager sharedManager] fileSetsPath], score.token, filename];
    
    ReaderDocument *document = [ReaderDocument withDocumentFilePath:filepath password:nil];
    if(document) {
        PRTWorkspaceViewController *workspaceViewController = [[PRTWorkspaceViewController alloc] initWithReaderDocument:document];
        workspaceViewController.room = room;
        workspaceViewController.currentSession = session;
        [self presentViewController:workspaceViewController animated:YES completion:^{
            if(owner) {
                [APPDELEGATE stopListening];
            }
        }];
    } else {
        PRT_ERROR_LOG(@"%s (unexpected error - cannot create ReaderDocument object)", __PRETTY_FUNCTION__);
    }
    
}

- (void)openWorkspaceInRoom:(Room *)room asOwner:(BOOL)owner {
    [self openWorkspaceInRoom:room withSession:nil asOwner:owner];
}

@end
