//
//  PRTWorkspaceViewController.m
//  Partiture
//
//  Created by Kaarel Kruus on 11/02/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "PRTWorkspaceViewController.h"
#import "PRTCommunicationManager.h"

#import "Session+Additions.h"
#import "Remark+Additions.h"
#import "Member+Additions.h"

#import "PRTAppDelegate.h"

#import "PRTDrawingView.h"

#import "ReaderContentView.h"

#import "PRTAlertView.h"

#import "PRTWorkspaceRemarkPopOver.h"

@interface PRTWorkspaceViewController ()

@property (nonatomic, assign) NSInteger fakePageTurnIndex;
@property (nonatomic, strong) UIPopoverController *popOver;

@end

@implementation PRTWorkspaceViewController

- (BOOL)drawingView:(PRTDrawingView *)drawingView shouldDrawRemarkView:(PRTRemarkView *)remarkView withUuid:(NSString *)uuid {
    //NSLog(@"%s", __PRETTY_FUNCTION__);
    return YES;
}

- (void)drawingView:(PRTDrawingView *)drawingView didDrawRemarkView:(PRTRemarkView *)remarkView withUuid:(NSString *)uuid {
    //NSLog(@"%s", __PRETTY_FUNCTION__);
    
    // a new remark (highlight) - save to database
    BOOL isPublicLayer = NO; // TODO: is private or public layer used?
    
    Member *member = [Member fetchMemberWithToken:[APPDELEGATE appUser].token];
    if(member) {
        
        Participation *participation = [self.currentSession participationForMember:member];
        if(participation) {
            
            Layer *layer = isPublicLayer ? [self.currentSession fetchPublicLayer] : [self.currentSession fetchPrivateLayer];
            if(layer) {
                
                Remark *remark;
                remark = [Remark remarkWithData:[PRTUtils dataFromObject:remarkView]
                                        forPage:drawingView.pageNumber
                                        onLayer:layer
                              withParticipation:participation];
                remark.uuid = uuid;
                
                [APPDELEGATE saveContext];
                
                // TODO: distribute!
                
            } else {
                PRT_ERROR_LOG(@"%s (unexpected error - missing layer)", __PRETTY_FUNCTION__);
            }
            
        } else {
            // TOOD: handle
        }
        
    } else {
        // TOOD: handle
    }
    
}

- (void)drawingView:(PRTDrawingView *)drawingView didLongPressOnRemarkView:(PRTRemarkView *)remarkView withUuid:(NSString *)uuid {
    //NSLog(@"%s", __PRETTY_FUNCTION__);
    
    PRTWorkspaceRemarkPopOver *contentViewController = [[PRTWorkspaceRemarkPopOver alloc] initWithCreatedDate:[NSDate date]
                                                                                                    createdBy:@"Kaarel Kruus"
                                                                                                      canMove:YES canDelete:YES]; // TODO: fetch this info
    [contentViewController setMoveBlock:^{
        
        DISPATCH_ASYNC_MAIN({
            
            // shake gesture
            //[remarkView shakeView];
            
            // close popover
            [self.popOver dismissPopoverAnimated:YES];
            
        });
    }];
    
    [contentViewController setDeleteBlock:^{
        
        DISPATCH_ASYNC_MAIN({
            
            // remove from UI
            UIView *view = [drawingView viewForUuid:uuid];
            [view removeFromSuperview];
            
            // remove from DB
            Remark *remark = [Remark fetchRemarkWithUuid:uuid];
            if(remark) {
                [[APPDELEGATE managedObjectContext] deleteObject:remark];
            }
            
            // close popover
            [self.popOver dismissPopoverAnimated:YES];
            
        });
    }];
    
    self.popOver = [[UIPopoverController alloc] initWithContentViewController:contentViewController];
    self.popOver.popoverContentSize = contentViewController.view.bounds.size;
    [self.popOver presentPopoverFromRect:remarkView.frame inView:drawingView permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
}

- (Membership *)publisher:(PRTCOMPublisher *)publisher membershipForMemberWithToken:(NSString *)token inRoom:(Room *)room {
    NSLog(@"%s (%@)", __PRETTY_FUNCTION__, token);
    Member *member = [Member fetchMemberWithToken:token];
    return [room fetchMemberShipOfMember:member];
}

- (BOOL)publisher:(PRTCOMPublisher *)publisher hasParticipatingMember:(Member *)member inSession:(Session *)session {
    NSLog(@"%s", __PRETTY_FUNCTION__);
    return [session participationForMember:member];
}

- (void)closeWorkspace {
    [self.currentSession saveAndEnd]; // TODO
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)tappedInToolbar:(ReaderMainToolbar *)toolbar closeButton:(UIButton *)button {
    NSLog(@"%s", __PRETTY_FUNCTION__);
    
    if(self.currentSession.isPublished) {
        PRTAlertView *alert = [[PRTAlertView alloc] init];
        alert.title = @"Attention!";
        alert.message = @"Are you sure you want to stop publishing the session and close the room?";
        [alert addButtonWithTitle:@"No"]; [alert addButtonWithTitle:@"Yes"];
        [alert setTappedButtonBlock:^(NSInteger index) {
            if(index == 1) { // Yes
                [self closeWorkspace];
            }
        }];
        [alert show];
    } else {
        [self closeWorkspace];
    }
}

- (void)tappedInToolbar:(ReaderMainToolbar *)toolbar syncButton:(UIButton *)button {
    NSLog(@"%s", __PRETTY_FUNCTION__);
}

- (void)tappedInToolbar:(ReaderMainToolbar *)toolbar pointButton:(UIButton *)button {
    NSLog(@"%s", __PRETTY_FUNCTION__);
}

- (void)tappedInToolbar:(ReaderMainToolbar *)toolbar privateEditButton:(UIButton *)button {
    NSLog(@"%s", __PRETTY_FUNCTION__);
}

- (void)tappedInToolbar:(ReaderMainToolbar *)toolbar publicEditButton:(UIButton *)button {
    NSLog(@"%s", __PRETTY_FUNCTION__);
}

- (void)tappedInToolbar:(ReaderMainToolbar *)toolbar publishButton:(UIButton *)button {
    NSLog(@"%s", __PRETTY_FUNCTION__);
    
    // end whatever already exists
    [self.currentSession saveAndEnd];
    
    if(self.currentSession.isPublished) {
        [self createAndAssignUnpublishedSession];
        button.selected = NO;
    } else {
        [self createAndAssignPublishedSession];
        button.selected = YES;
    }
}

- (void)createAndAssignUnpublishedSession {
    if(self.room) {
        Session *session = [Session createAndStartUnpublishedInRoom:self.room];
        self.currentSession = session;
    } else {
        // TODO: handle?
    }
}

- (void)createAndAssignPublishedSession {
    if(self.room) {
        Session *session = [Session createAndStartPublishedInRoom:self.room];
        self.currentSession = session;
        
        [NSTimer scheduledTimerWithTimeInterval:5.0F
                                         target:self
                                       selector:@selector(fakePageTurn:)
                                       userInfo:nil
                                        repeats:YES]; // TODO: remove
        
    } else {
        // TODO: handle?
    }
}

- (void)fakePageTurn:(NSTimer *)timer {
    [self.currentSession pageTurn:self.fakePageTurnIndex];
    self.fakePageTurnIndex++;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    COM_PUBLISHER.delegate = self;
    
    if(!self.currentSession) {
        // automatically create a non-shared session
        [self createAndAssignUnpublishedSession];
    } else {
        // TODO: what if session resumed after i.e. crash?
        NSLog(@"Resume the session!");
    }
}

- (NSArray *)remarksInRoom:(Room *)room forPage:(NSInteger)page onPublicLayer:(BOOL)onPublicLayer {
    NSMutableArray *remarks = [NSMutableArray array];
    for (Session *session in room.sessions) {
        NSArray *remarksInSession = (onPublicLayer ? [[[session fetchPublicLayer] remarks] allObjects] : [[[session fetchPrivateLayer] remarks] allObjects]);
        for (Remark *remark in remarksInSession) {
            if(remark.page.integerValue == page) {
                [remarks addObject:remark];
            }
        }
    }
    return remarks;
}

- (void)readyToRenderOnReaderContentView:(ReaderContentView *)readerContentView {
    NSInteger pageIndex = readerContentView.page;
    [readerContentView renderRemarks:[self remarksInRoom:self.room forPage:pageIndex onPublicLayer:NO]];
    //[readerContentView renderRemarks:[self remarksInRoom:self.room forPage:pageIndex onPublicLayer:YES]];
}

@end
