//
//  PRTWorkspaceViewController.m
//  Partiture
//
//  Created by Kaarel Kruus on 11/02/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "PRTWorkspaceViewController.h"
#import "PRTAppDelegate.h"
#import "Annotation.h"
#import <MBProgressHUD/MBProgressHUD.h>

@interface PRTWorkspaceViewController ()

@end

@implementation PRTWorkspaceViewController

- (void)saveOrUpdate:(Annotation *)annotation {
    
    Annotation *db_annotation = [self getAnnotationByGuid:annotation.guid];
    if(db_annotation) { // already in DB
        // just update (most probably moved annotation)
        NSLog(@"UPDATE OLD ANNOTATION!");
        db_annotation.data = annotation.data;
        [APPDELEGATE saveContext];
    } else {
        NSLog(@"SAVE NEW ANNOTATION!");
        [[APPDELEGATE managedObjectContext] insertObject:annotation];
        [APPDELEGATE saveContext];
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self addAnnotations:[NSArray arrayWithObject:annotation]];
    });
}

- (Annotation *)getAnnotationByGuid:(NSString *)guid {
    
    NSManagedObjectContext *context = [APPDELEGATE managedObjectContext];
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Annotation" inManagedObjectContext:context];
    [fetchRequest setEntity:entity];
    
    NSError *error = nil;
    
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"guid == %@", guid];
    fetchRequest.predicate = predicate;
    
    NSArray *fetchedObjects = [context executeFetchRequest:fetchRequest error:&error];
    return (fetchedObjects.count > 0) ? fetchedObjects[0] : nil;
}

@end

@implementation PRTAdvertiserController

- (void)pageTurn:(NSInteger)index {
    //NSLog(@"Page turn: %i", index);
    
    NSDictionary *d = [NSDictionary dictionaryWithObjectsAndKeys:@(index), @"pageTurn", nil];
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:d];
    
    for (MCSession *session in self.sessions) {
        [session sendData:data
                  toPeers:session.connectedPeers
                 withMode:MCSessionSendDataReliable
                    error:nil];
    }
}

- (void)onAnnotation:(Annotation *)annotation forGuid:(NSString *)guid {
    
    NSDictionary *d = [NSDictionary dictionaryWithObjectsAndKeys:annotation, @"annotation", nil];
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:d];
    
    for (MCSession *session in self.sessions) {
        [session sendData:data
                  toPeers:session.connectedPeers
                 withMode:MCSessionSendDataReliable
                    error:nil];
    }
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.sessions = [[NSMutableArray alloc] init];
    
    self.connectedPeers = [[NSMutableArray alloc] init];
    self.notConnectedPeers = [[NSMutableArray alloc] init];
    self.connectingPeers = [[NSMutableArray alloc] init];
    
    NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init];
    [dateFormat setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en"]];
    [dateFormat setDateFormat: @"dd.MM.yyyy HH:mm:ss.SSSSSS ZZZ"];
    NSString *currentTimestamp = [dateFormat stringFromDate:[NSDate date]];
    
    self.peerID = [[MCPeerID alloc] initWithDisplayName:[NSString stringWithFormat:@"Master (%@)", currentTimestamp]];
    
    self.serviceAdvertiser = [[MCNearbyServiceAdvertiser alloc] initWithPeer:self.peerID
                                                               discoveryInfo:nil
                                                                 serviceType:@"partiture"];
    self.serviceAdvertiser.delegate = self;
    
    [self.serviceAdvertiser startAdvertisingPeer];
}

- (void)advertiser:(MCNearbyServiceAdvertiser *)advertiser didNotStartAdvertisingPeer:(NSError *)error {
    NSLog(@"- - - didNotStartAdvertisingPeer: %@", error);
}

- (void)advertiser:(MCNearbyServiceAdvertiser *)advertiser didReceiveInvitationFromPeer:(MCPeerID *)peerID
       withContext:(NSData *)context
 invitationHandler:(void (^)(BOOL, MCSession *))invitationHandler {
    NSLog(@"- - - didReceiveInvitationFromPeer: (%@)", peerID.displayName);
    
    // AUTOMATICALLY ACCEPT INVITATION
    MCSession *session = [self availableSession];
    invitationHandler(YES, session);
}

- (MCSession *)availableSession {
    
    //Try and use an existing session (_sessions is a mutable array)
    //for (MCSession *session in _sessions)
    //if ([session.connectedPeers count] < 1)
    //return session;
    
    //Or create a new session
    MCSession *newSession = [self newSession];
    [_sessions addObject:newSession];
    
    return newSession;
}

- (MCSession *)newSession {
    NSLog(@"* * * CREATE NEW SESSION! * * *");
    MCSession *session = [[MCSession alloc] initWithPeer:self.peerID securityIdentity:nil encryptionPreference:MCEncryptionNone];
    session.delegate = self;
    return session;
}

- (void)session:(MCSession *)session didReceiveData:(NSData *)data fromPeer:(MCPeerID *)peerID {
    NSLog(@"- - - didReceiveData: %@", data);
    
    id package = [NSKeyedUnarchiver unarchiveObjectWithData:data];
    
    if([package isKindOfClass:[NSDictionary class]]) {
        if([((NSDictionary *)package) objectForKey:@"pageTurn"]) {
            int index = [[((NSDictionary *)package) objectForKey:@"pageTurn"] intValue];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self showDocumentPage:index];
            });
        } else if([((NSDictionary *)package) objectForKey:@"annotation"]) {
            Annotation *annotation = [((NSDictionary *)package) objectForKey:@"annotation"];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self saveOrUpdate:annotation];
            });
        }
    } else if([package isKindOfClass:[NSArray class]]) {
        
    }
}

- (void)session:(MCSession *)session didReceiveStream:(NSInputStream *)stream withName:(NSString *)streamName fromPeer:(MCPeerID *)peerID {
    NSLog(@"- - - didReceiveStream");
}

- (void)session:(MCSession *)session didStartReceivingResourceWithName:(NSString *)resourceName fromPeer:(MCPeerID *)peerID withProgress:(NSProgress *)progress {
    NSLog(@"- - - didStartReceivingResourceWithName");
}

- (void)session:(MCSession *)session didFinishReceivingResourceWithName:(NSString *)resourceName fromPeer:(MCPeerID *)peerID atURL:(NSURL *)localURL withError:(NSError *)error {
    NSLog(@"- - - didFinishReceivingResourceWithName");
}

- (void)session:(MCSession *)session peer:(MCPeerID *)peerID didChangeState:(MCSessionState)state {
    
    NSLog(@"- - - didChangeState: %i", state);
    
    if(state == MCSessionStateConnected) {
        
        NSLog(@"%@ is connected", peerID);
        
        [self.connectingPeers removeObject:peerID];
        [self.notConnectedPeers removeObject:peerID];
        [self.connectedPeers addObject:peerID];
        
        //[self sync];
        //[self.serviceAdvertiser stopAdvertisingPeer];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
            hud.labelText = @"Connected";
            [hud hide:YES afterDelay:2.0F];
        });
        
    } else if(state == MCSessionStateNotConnected) {
        
        NSLog(@"%@ is not connected", peerID);
        
        [self.connectingPeers removeObject:peerID];
        [self.connectedPeers removeObject:peerID];
        [self.notConnectedPeers addObject:peerID];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
            hud.labelText = @"Disconnected";
            [hud hide:YES afterDelay:2.0F];
        });
        
    } else {
        
        NSLog(@"%@ is connecting", peerID);
        
        [self.notConnectedPeers removeObject:peerID];
        [self.connectedPeers removeObject:peerID];
        [self.connectingPeers addObject:peerID];
    }
}

- (void)sync {
    
    NSLog(@"SYNC UP!");
    
    /*
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:[self getAnnotations]];
    
    for (MCSession *session in self.sessions) {
        [session sendData:data
                  toPeers:session.connectedPeers
                 withMode:MCSessionSendDataReliable
                    error:nil];
    }
     */
}

- (NSArray *)getAnnotations {
    
    NSManagedObjectContext *context = [APPDELEGATE managedObjectContext];
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Annotation"
                                              inManagedObjectContext:context];
    [fetchRequest setEntity:entity];
    
    NSError *error = nil;
    
    NSArray *fetchedObjects = [context executeFetchRequest:fetchRequest error:&error];
    return fetchedObjects;
}

@end

@implementation PRTBrowserController

- (void)pageTurn:(NSInteger)index {
    //NSLog(@"Page turn: %i", index);
    
    NSDictionary *d = [NSDictionary dictionaryWithObjectsAndKeys:@(index), @"pageTurn", nil];
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:d];
    
    [self.session sendData:data
              toPeers:self.session.connectedPeers
             withMode:MCSessionSendDataReliable
                error:nil];
}

- (void)onAnnotation:(Annotation *)annotation forGuid:(NSString *)guid {
    
    NSDictionary *d = [NSDictionary dictionaryWithObjectsAndKeys:annotation, @"annotation", nil];
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:d];
    
    [self.session sendData:data
                   toPeers:self.session.connectedPeers
                  withMode:MCSessionSendDataReliable
                     error:nil];
}

- (void)browser:(MCNearbyServiceBrowser *)browser foundPeer:(MCPeerID *)peerID withDiscoveryInfo:(NSDictionary *)info {
    NSLog(@"- - - foundPeer (%@)", peerID.displayName);
    
    // AUTOMATICALLY SEND INVITATION
    [self.serviceBrowser invitePeer:peerID toSession:self.session withContext:nil timeout:10.0F];
}

- (void)browser:(MCNearbyServiceBrowser *)browser lostPeer:(MCPeerID *)peerID {
    NSLog(@"- - - lostPeer (%@)", peerID.displayName);
}

- (void)session:(MCSession *)session didReceiveData:(NSData *)data fromPeer:(MCPeerID *)peerID {
    NSLog(@"- - - didReceiveData: %@", data);
    
    /*
    NSArray *annotations = (NSArray *)[NSKeyedUnarchiver unarchiveObjectWithData:data];
    //NSLog(@"* * * Received: %@", annotations);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self addAnnotations:annotations];
    });
     */
    
    id package = [NSKeyedUnarchiver unarchiveObjectWithData:data];
    
    if([package isKindOfClass:[NSDictionary class]]) {
        if([((NSDictionary *)package) objectForKey:@"pageTurn"]) {
            int index = [[((NSDictionary *)package) objectForKey:@"pageTurn"] intValue];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self showDocumentPage:index];
            });
        } else if([((NSDictionary *)package) objectForKey:@"annotation"]) {
            Annotation *annotation = [((NSDictionary *)package) objectForKey:@"annotation"];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self saveOrUpdate:annotation];
            });
        }
    } else if([package isKindOfClass:[NSArray class]]) {
        
    }
}

- (void)session:(MCSession *)session didReceiveStream:(NSInputStream *)stream withName:(NSString *)streamName fromPeer:(MCPeerID *)peerID {
    NSLog(@"- - - didReceiveStream");
}

- (void)session:(MCSession *)session didStartReceivingResourceWithName:(NSString *)resourceName fromPeer:(MCPeerID *)peerID withProgress:(NSProgress *)progress {
    NSLog(@"- - - didStartReceivingResourceWithName: %@", resourceName);
}

- (void)session:(MCSession *)session didFinishReceivingResourceWithName:(NSString *)resourceName fromPeer:(MCPeerID *)peerID atURL:(NSURL *)localURL withError:(NSError *)error {
    NSLog(@"- - - didFinishReceivingResourceWithName: %@", resourceName);
}

- (void)session:(MCSession *)session peer:(MCPeerID *)peerID didChangeState:(MCSessionState)state {
    NSLog(@"- - - didChangeState: %i", state);
        
    if(state == MCSessionStateConnected) {
        NSLog(@"%@ is connected", peerID);
        
        //[self.serviceBrowser stopBrowsingForPeers];
        
        dispatch_async(dispatch_get_main_queue(), ^{
            MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
            hud.labelText = @"Connected";
            [hud hide:YES afterDelay:2.0F];
        });
        
    } else if(state == MCSessionStateNotConnected) {
        NSLog(@"%@ is not connected", peerID);
        
        dispatch_async(dispatch_get_main_queue(), ^{
            MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
            hud.labelText = @"Disconnected";
            [hud hide:YES afterDelay:2.0F];
        });
        
    } else {
        NSLog(@"%@ is connecting", peerID);
    }
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    MCPeerID *peerID = [[MCPeerID alloc] initWithDisplayName:@"Slave"];
    self.session = [[MCSession alloc] initWithPeer:peerID];
    self.session.delegate = self;
    
    self.serviceBrowser = [[MCNearbyServiceBrowser alloc] initWithPeer:peerID serviceType:@"partiture"];
    self.serviceBrowser.delegate = self;
    
    [self.serviceBrowser startBrowsingForPeers];
}

- (id)initWithReaderDocument:(ReaderDocument *)object
{
    self = [super initWithReaderDocument:object];
    if (self) {
        self.availablePeers = [NSMutableArray array];
    }
    return self;
}

@end
