//
//  Room+Additions.m
//  Partiture
//
//  Created by Kaarel Kruus on 16/03/14.
//  Copyright (c) 2014 Kaarel Kruus. All rights reserved.
//

#import "Room+Additions.h"
#import "PRTAppDelegate.h"
#import "PRTUtils.h"
#import "Membership+Enums.h"
#import "Membership+Additions.h"
#import "Member+Additions.h"

@implementation Room (Additions)

+ (Room *)roomWithName:(NSString *)name uuid:(NSString *)uuid andPasskey:(NSString *)passkey forBundle:(Bundle *)bundle {
    
    Room *newRoom = [NSEntityDescription insertNewObjectForEntityForName:NSStringFromClass(Room.class)
                                                  inManagedObjectContext:[APPDELEGATE managedObjectContext]];
    newRoom.uuid = (uuid && uuid.length > 0) ? uuid : [PRTUtils generateUUID];
    newRoom.name = name; // TODO: allow to create 2+ more rooms with the same name?
    newRoom.passkey = passkey;
    
    // bind
    newRoom.bundle = bundle;
    
    [APPDELEGATE saveContext];
    
    // add the current user as owner or guest to the room
    Member *member = [Member memberWithName:[APPDELEGATE appUser].name
                                   andToken:[APPDELEGATE appUser].token];
    
    Membership *memberShip = [Membership membershipWithMember:member inRoom:newRoom];
    memberShip.role = (!uuid) ? @(MembershipRoleRoomOwner) : @(MembershipRoleRoomGuest);
    
    [APPDELEGATE saveContext];
    
    return newRoom;
}

+ (Room *)roomWithName:(NSString *)name andPasskey:(NSString *)passkey forBundle:(Bundle *)bundle {
    return [self roomWithName:name uuid:nil andPasskey:passkey forBundle:bundle];
}

- (Membership *)fetchMemberShipOfMember:(Member *)member {
    // iterate over the membership entities
    NSArray *memberships = self.members.allObjects;
    for(Membership *membership in memberships) {
        if(membership.member == member) {
            return membership;
        }
    }
    return nil;
}

@end
